# This is spectacularly generated code by spectacular based on
# Qlik Cloud Services APIs

from __future__ import annotations

from dataclasses import asdict, dataclass
from typing import List, Literal

from ..auth import Auth, Config


@dataclass
class Email:
    """

    Attributes
    ----------
    body: str
      email body
    recipient: str
      email recipient (email address)
    subject: str
      email subject
    """

    body: str = None
    recipient: str = None
    subject: str = None

    def __init__(self_, **kvargs):
        if "body" in kvargs and kvargs["body"] is not None:
            self_.body = kvargs["body"]
        if "recipient" in kvargs and kvargs["recipient"] is not None:
            self_.recipient = kvargs["recipient"]
        if "subject" in kvargs and kvargs["subject"] is not None:
            self_.subject = kvargs["subject"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class EmailConfigFieldPatch:
    """
    A JSON Patch document as defined in https://datatracker.ietf.org/doc/html/rfc6902.

    Attributes
    ----------
    op: Literal["replace, remove, add"]
      The operation to be performed.
    path: Literal["/username", "/serverAddress", "/serverPort", "/securityType", "/emailAddress", "/emailPassword"]
      The path for the given resource field to patch.
    value: str
      The value to be used for this operation.
    """

    op: Literal["replace, remove, add"] = None
    path: Literal[
        "/username",
        "/serverAddress",
        "/serverPort",
        "/securityType",
        "/emailAddress",
        "/emailPassword",
    ] = None
    value: str = None

    def __init__(self_, **kvargs):
        if "op" in kvargs and kvargs["op"] is not None:
            self_.op = kvargs["op"]
        if "path" in kvargs and kvargs["path"] is not None:
            self_.path = kvargs["path"]
        if "value" in kvargs and kvargs["value"] is not None:
            self_.value = kvargs["value"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class EmailConfigGet:
    """

    Attributes
    ----------
    authFailures: float
      Number of authentication failures
    emailAddress: str
      used for SMTP authentication
    isValid: bool
      Is the configuration valid
    modificationTime: str
      Last modification time. Formatted as a ISO 8601 string.
    passwordExists: bool
      Indicates if password is defined for this smtp config. The password itself is not returned!
    securityType: str
      one of none, StartTLS or SSL/TLS
    serverAddress: str
      domain name or IP address of SMTP server
    serverPort: float
      smtp server listening port
    status: SmtpConfigStatus
      Contains statusCode and statusReason
    tenantId: str
      The tenant Id
    username: str
      user name
    """

    authFailures: float = None
    emailAddress: str = None
    isValid: bool = None
    modificationTime: str = None
    passwordExists: bool = None
    securityType: str = None
    serverAddress: str = None
    serverPort: float = None
    status: SmtpConfigStatus = None
    tenantId: str = None
    username: str = None

    def __init__(self_, **kvargs):
        if "authFailures" in kvargs and kvargs["authFailures"] is not None:
            self_.authFailures = kvargs["authFailures"]
        if "emailAddress" in kvargs and kvargs["emailAddress"] is not None:
            self_.emailAddress = kvargs["emailAddress"]
        if "isValid" in kvargs and kvargs["isValid"] is not None:
            self_.isValid = kvargs["isValid"]
        if "modificationTime" in kvargs and kvargs["modificationTime"] is not None:
            self_.modificationTime = kvargs["modificationTime"]
        if "passwordExists" in kvargs and kvargs["passwordExists"] is not None:
            self_.passwordExists = kvargs["passwordExists"]
        if "securityType" in kvargs and kvargs["securityType"] is not None:
            self_.securityType = kvargs["securityType"]
        if "serverAddress" in kvargs and kvargs["serverAddress"] is not None:
            self_.serverAddress = kvargs["serverAddress"]
        if "serverPort" in kvargs and kvargs["serverPort"] is not None:
            self_.serverPort = kvargs["serverPort"]
        if "status" in kvargs and kvargs["status"] is not None:
            if (
                type(kvargs["status"]).__name__
                == EmailConfigGet.__annotations__["status"]
            ):
                self_.status = kvargs["status"]
            else:
                self_.status = SmtpConfigStatus(**kvargs["status"])
        if "tenantId" in kvargs and kvargs["tenantId"] is not None:
            self_.tenantId = kvargs["tenantId"]
        if "username" in kvargs and kvargs["username"] is not None:
            self_.username = kvargs["username"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class EmailConfigPatch(List["EmailConfigFieldPatch"]):
    """

    Attributes
    ----------
    """

    def __init__(self_, elements):
        for e in elements:
            self_.append(EmailConfigFieldPatch(**e))


@dataclass
class SmtpCheck:
    """

    Attributes
    ----------
    isValid: bool
      true if smtp config is correct and complete. Will return false if smtp-config does not exist at all
    """

    isValid: bool = None

    def __init__(self_, **kvargs):
        if "isValid" in kvargs and kvargs["isValid"] is not None:
            self_.isValid = kvargs["isValid"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class SmtpConfigStatus:
    """
    Contains statusCode and statusReason

    Attributes
    ----------
    statusCode: float
      Status code
    statusReason: str
      Status reason
    """

    statusCode: float = None
    statusReason: str = None

    def __init__(self_, **kvargs):
        if "statusCode" in kvargs and kvargs["statusCode"] is not None:
            self_.statusCode = kvargs["statusCode"]
        if "statusReason" in kvargs and kvargs["statusReason"] is not None:
            self_.statusReason = kvargs["statusReason"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class SmtpResult:
    """

    Attributes
    ----------
    connectionFailed: bool
      could not resolve domain name, connection refused, connection timed out, SSL mismatch
    message: str
      error message from SMTP middleware .. a bit technical but could be useful to administrator
    smtpResponseCode: int
      smtp result code string from the SMTP server. eg. "250 2.6.0"
    success: bool
      was SMTP operation successful or not. Other fields herein provide more detail
    """

    connectionFailed: bool = None
    message: str = None
    smtpResponseCode: int = None
    success: bool = None

    def __init__(self_, **kvargs):
        if "connectionFailed" in kvargs and kvargs["connectionFailed"] is not None:
            self_.connectionFailed = kvargs["connectionFailed"]
        if "message" in kvargs and kvargs["message"] is not None:
            self_.message = kvargs["message"]
        if "smtpResponseCode" in kvargs and kvargs["smtpResponseCode"] is not None:
            self_.smtpResponseCode = kvargs["smtpResponseCode"]
        if "success" in kvargs and kvargs["success"] is not None:
            self_.success = kvargs["success"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Transports:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def send_test_email_email_config(self, data: Email) -> SmtpResult:
        """
        Send a test mail with the supplied email info (subject, body, recipient). Email config from database is used for the connection.

        Parameters
        ----------
        data: Email
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/transports/email-config/actions/send-test-email",
            method="POST",
            params={},
            data=data,
        )
        obj = SmtpResult(**response.json())
        obj.auth = self.auth
        return obj

    def validate_email_config(self) -> SmtpCheck:
        """
        Returns the isValid value for the email configuration for the tenant. Will return false if no email configuration exists.

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/transports/email-config/actions/validate",
            method="POST",
            params={},
            data=None,
        )
        obj = SmtpCheck(**response.json())
        obj.auth = self.auth
        return obj

    def verify_connection_email_config(self) -> SmtpResult:
        """
        Verifies connection to email server for tenant provided via JWT

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/transports/email-config/actions/verify-connection",
            method="POST",
            params={},
            data=None,
        )
        obj = SmtpResult(**response.json())
        obj.auth = self.auth
        return obj

    def delete_email_config(self) -> None:
        """
        Deletes email configuration for a given tenant id (retrieved from JWT).

        Parameters
        ----------
        """
        self.auth.rest(
            path="/transports/email-config",
            method="DELETE",
            params={},
            data=None,
        )

    def get_email_config(self) -> EmailConfigGet:
        """
        Returns the email configuration for a given tenant id (retrieved from JWT).

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/transports/email-config",
            method="GET",
            params={},
            data=None,
        )
        obj = EmailConfigGet(**response.json())
        obj.auth = self.auth
        return obj

    def patch_email_config(self, data: EmailConfigPatch) -> None:
        """
        Patch the email configuration for a given tenant id (retrieved from JWT).

        Parameters
        ----------
        data: EmailConfigPatch
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        self.auth.rest(
            path="/transports/email-config",
            method="PATCH",
            params={},
            data=data,
        )



# This is spectacularly generated code by spectacular based on
# Qlik Cloud Services APIs

from __future__ import annotations

from dataclasses import asdict, dataclass
from typing import List, Literal

from ..auth import Auth, Config


@dataclass
class Tenant:
    """

    Attributes
    ----------
    autoAssignCreateSharedSpacesRoleToProfessionals: bool
    autoAssignDataServicesContributorRoleToProfessionals: bool
    autoAssignPrivateAnalyticsContentCreatorRoleToProfessionals: bool
    created: str
      The timestamp for when the tenant record was created (1970-01-01T00:00:00.001Z for static tenants).
    createdByUser: str
      The user ID who created the tenant.
    datacenter: str
      The datacenter where the tenant is located.
    enableAnalyticCreation: bool
    hostnames: list[str]
      List of case insensitive hostnames that are mapped to the tenant. The first record maps to the display name and the subsequent entries are aliases.
    id: str
      The unique tenant identifier.
    lastUpdated: str
      The timestamp for when the tenant record was last updated (1970-01-01T00:00:00.001Z for static tenants).
    links: TenantLinks
    name: str
      The display name of the tenant.
    status: Literal["active", "disabled", "deleted"]
      The status of the tenant.
    """

    autoAssignCreateSharedSpacesRoleToProfessionals: bool = True
    autoAssignDataServicesContributorRoleToProfessionals: bool = True
    autoAssignPrivateAnalyticsContentCreatorRoleToProfessionals: bool = True
    created: str = None
    createdByUser: str = None
    datacenter: str = None
    enableAnalyticCreation: bool = None
    hostnames: list[str] = None
    id: str = None
    lastUpdated: str = None
    links: TenantLinks = None
    name: str = None
    status: Literal["active", "disabled", "deleted"] = None

    def __init__(self_, **kvargs):
        if (
            "autoAssignCreateSharedSpacesRoleToProfessionals" in kvargs
            and kvargs["autoAssignCreateSharedSpacesRoleToProfessionals"] is not None
        ):
            self_.autoAssignCreateSharedSpacesRoleToProfessionals = kvargs[
                "autoAssignCreateSharedSpacesRoleToProfessionals"
            ]
        if (
            "autoAssignDataServicesContributorRoleToProfessionals" in kvargs
            and kvargs["autoAssignDataServicesContributorRoleToProfessionals"]
            is not None
        ):
            self_.autoAssignDataServicesContributorRoleToProfessionals = kvargs[
                "autoAssignDataServicesContributorRoleToProfessionals"
            ]
        if (
            "autoAssignPrivateAnalyticsContentCreatorRoleToProfessionals" in kvargs
            and kvargs["autoAssignPrivateAnalyticsContentCreatorRoleToProfessionals"]
            is not None
        ):
            self_.autoAssignPrivateAnalyticsContentCreatorRoleToProfessionals = kvargs[
                "autoAssignPrivateAnalyticsContentCreatorRoleToProfessionals"
            ]
        if "created" in kvargs and kvargs["created"] is not None:
            self_.created = kvargs["created"]
        if "createdByUser" in kvargs and kvargs["createdByUser"] is not None:
            self_.createdByUser = kvargs["createdByUser"]
        if "datacenter" in kvargs and kvargs["datacenter"] is not None:
            self_.datacenter = kvargs["datacenter"]
        if (
            "enableAnalyticCreation" in kvargs
            and kvargs["enableAnalyticCreation"] is not None
        ):
            self_.enableAnalyticCreation = kvargs["enableAnalyticCreation"]
        if "hostnames" in kvargs and kvargs["hostnames"] is not None:
            self_.hostnames = kvargs["hostnames"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "lastUpdated" in kvargs and kvargs["lastUpdated"] is not None:
            self_.lastUpdated = kvargs["lastUpdated"]
        if "links" in kvargs and kvargs["links"] is not None:
            if type(kvargs["links"]).__name__ == Tenant.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = TenantLinks(**kvargs["links"])
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "status" in kvargs and kvargs["status"] is not None:
            self_.status = kvargs["status"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)

    def deactivate(
        self, qlik_confirm_hostname: str, data: TenantDeactivateRequest = None
    ) -> TenantDeactivateResponse:
        """
        Deactivate a tenant
        Deactivates a tenant.

        Parameters
        ----------
        data: TenantDeactivateRequest = None
          A request to deactivate a tenant.
        qlik_confirm_hostname: str
          A confirmation string that should match the hostname associated with the tenant resource to be deactivated. Example: unicorn.eu.qlikcloud.com
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        headers = {}
        if qlik_confirm_hostname:
            headers["qlik-confirm-hostname"] = qlik_confirm_hostname
        response = self.auth.rest(
            path="/tenants/{tenantId}/actions/deactivate".replace(
                "{tenantId}", self.id
            ),
            method="POST",
            params={},
            data=data,
            headers=headers,
        )
        obj = TenantDeactivateResponse(**response.json())
        obj.auth = self.auth
        return obj

    def reactivate(self, qlik_confirm_hostname: str) -> object:
        """
        Reactivates a tenant
        Reactivates a disabled tenant.

        Parameters
        ----------
        qlik_confirm_hostname: str
          A confirmation string that should match one of the hostnames of the tenant resource to be reactivated. Example: unicorn.eu.qlikcloud.com
        """
        headers = {}
        if qlik_confirm_hostname:
            headers["qlik-confirm-hostname"] = qlik_confirm_hostname
        response = self.auth.rest(
            path="/tenants/{tenantId}/actions/reactivate".replace(
                "{tenantId}", self.id
            ),
            method="POST",
            params={},
            data=None,
            headers=headers,
        )
        return response.json()

    def patch(self, data: TenantPatchSchema) -> None:
        """
        Update a tenant by id.

        Parameters
        ----------
        data: TenantPatchSchema
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        self.auth.rest(
            path="/tenants/{tenantId}".replace("{tenantId}", self.id),
            method="PATCH",
            params={},
            data=data,
        )


@dataclass
class TenantCreationRequest:
    """

    Attributes
    ----------
    datacenter: str
      The datacenter where the tenant is located.
    hostnames: list[str]
      The hostnames of the created tenant. Can only create with a single entry that lines up wtih the tenant name.
    licenseKey: str
      The signed license key of the license that will be associated with the created tenant.
    name: str
      The name of the created tenant, provided by the onboarding service.
    """

    datacenter: str = None
    hostnames: list[str] = None
    licenseKey: str = None
    name: str = None

    def __init__(self_, **kvargs):
        if "datacenter" in kvargs and kvargs["datacenter"] is not None:
            self_.datacenter = kvargs["datacenter"]
        if "hostnames" in kvargs and kvargs["hostnames"] is not None:
            self_.hostnames = kvargs["hostnames"]
        if "licenseKey" in kvargs and kvargs["licenseKey"] is not None:
            self_.licenseKey = kvargs["licenseKey"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class TenantDeactivateRequest:
    """
    A request to deactivate a tenant.

    Attributes
    ----------
    purgeAfterDays: int
      Sets the number of days to purge the tenant after deactivation. Only available to OEMs.
    """

    purgeAfterDays: int = 30

    def __init__(self_, **kvargs):
        if "purgeAfterDays" in kvargs and kvargs["purgeAfterDays"] is not None:
            self_.purgeAfterDays = kvargs["purgeAfterDays"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class TenantDeactivateResponse:
    """
    The result of tenant deactivation.

    Attributes
    ----------
    estimatedPurgeDate: str
      The estimated date time of when tenant will be purged.
    id: str
      The unique tenant identifier.
    status: Literal["disabled"]
      The status of the tenant.
    """

    estimatedPurgeDate: str = None
    id: str = None
    status: Literal["disabled"] = None

    def __init__(self_, **kvargs):
        if "estimatedPurgeDate" in kvargs and kvargs["estimatedPurgeDate"] is not None:
            self_.estimatedPurgeDate = kvargs["estimatedPurgeDate"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "status" in kvargs and kvargs["status"] is not None:
            self_.status = kvargs["status"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class TenantLinks:
    """

    Attributes
    ----------
    self: TenantLinksSelf
      A link to this tenant.
    """

    self: TenantLinksSelf = None

    def __init__(self_, **kvargs):
        if "self" in kvargs and kvargs["self"] is not None:
            if type(kvargs["self"]).__name__ == TenantLinks.__annotations__["self"]:
                self_.self = kvargs["self"]
            else:
                self_.self = TenantLinksSelf(**kvargs["self"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class TenantLinksSelf:
    """
    A link to this tenant.

    Attributes
    ----------
    href: str
      URL that defines the resource.
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class TenantPatchSchema(List["TenantPatchSchemaElement"]):
    """

    Attributes
    ----------
    """

    def __init__(self_, elements):
        for e in elements:
            self_.append(TenantPatchSchemaElement(**e))


@dataclass
class TenantPatchSchemaElement:
    """
    A JSON Patch document as defined in http://tools.ietf.org/html/rfc6902.

    Attributes
    ----------
    op: Literal["replace"]
      The operation to be performed.
    path: Literal["/name", "/hostnames/1", "/autoAssignCreateSharedSpacesRoleToProfessionals", "/autoAssignPrivateAnalyticsContentCreatorRoleToProfessionals", "/autoAssignDataServicesContributorRoleToProfessionals", "/enableAnalyticCreation"]
      A JSON Pointer value that references a location within the target document where the operation is performed.
    value: object
      The value to be used for this operation.
    """

    op: Literal["replace"] = None
    path: Literal[
        "/name",
        "/hostnames/1",
        "/autoAssignCreateSharedSpacesRoleToProfessionals",
        "/autoAssignPrivateAnalyticsContentCreatorRoleToProfessionals",
        "/autoAssignDataServicesContributorRoleToProfessionals",
        "/enableAnalyticCreation",
    ] = None
    value: str | bool = None

    def __init__(self_, **kvargs):
        if "op" in kvargs and kvargs["op"] is not None:
            self_.op = kvargs["op"]
        if "path" in kvargs and kvargs["path"] is not None:
            self_.path = kvargs["path"]
        if "value" in kvargs and kvargs["value"] is not None:
            self_.value = kvargs["value"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Tenants:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def get_me(self) -> Tenant:
        """
        Redirects to current tenant.

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/tenants/me",
            method="GET",
            params={},
            data=None,
        )
        obj = Tenant(**response.json())
        obj.auth = self.auth
        return obj

    def get(self, tenantId: str) -> Tenant:
        """
        Retrieve a single tenant by id.

        Parameters
        ----------
        tenantId: str
          The id of the tenant to retrieve
        """
        response = self.auth.rest(
            path="/tenants/{tenantId}".replace("{tenantId}", tenantId),
            method="GET",
            params={},
            data=None,
        )
        obj = Tenant(**response.json())
        obj.auth = self.auth
        return obj

    def create(self, data: TenantCreationRequest) -> Tenant:
        """
        Creates a Tenant

        Parameters
        ----------
        data: TenantCreationRequest
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/tenants",
            method="POST",
            params={},
            data=data,
        )
        obj = Tenant(**response.json())
        obj.auth = self.auth
        return obj



import re


class RetryResponseInterceptor:
    def __init__(self, rpc):
        self._rpc = rpc

    def intercept(self, res):
        request_data = res["request_data"]
        # request_data = req
        # retry on request_aborted for get methods, except GetHyperCube and GetListObject
        # - exclude GetHyperCube* and GetListObject*.
        # - if they are aborted a new layout need to be fetched first to know if retrying the same parameter is relevant
        is_get_method = bool(
            re.match(
                r"(?!^GetHyperCube|^GetListObject)(^Get)",
                request_data.method,
                flags=re.IGNORECASE,
            )
        )
        request_aborted_code = 15
        is_request_aborted = (
            "error" in res and res["error"]["code"] == request_aborted_code
        )
        retry_allowed = is_get_method and is_request_aborted
        if "result" in res or not retry_allowed:
            return res
        return self._rpc.send(request_data)



# This is spectacularly generated code by spectacular based on
# Qlik Cloud Services APIs

from __future__ import annotations

import warnings
from dataclasses import asdict, dataclass
from typing import List, Literal

from ..auth import Auth, Config
from ..listable import ListableResource


@dataclass
class User:
    """
    A user object.

    Attributes
    ----------
    assignedGroups: list[UserAssignedGroups]
      An array of group references.
    assignedRoles: list[UserAssignedRoles]
      An array of role references.
    created: str
      Deprecated. Use `createdAt` instead.
    createdAt: str
      The timestamp for when the user record was created.
    email: str
      The email address for the user.
    id: str
      The unique user identifier.
    inviteExpiry: float
      The number of seconds until the user invitation will expire.
    lastUpdated: str
      Deprecated. Use `lastUpdatedAt` instead.
    lastUpdatedAt: str
      The timestamp for when the user record was last updated.
    links: UserLinks
      Pagination links to the user.
    locale: str
      Represents the end-user's language tag.
    name: str
      The name of the user.
    picture: str
      A static url linking to the avatar of the user.
    preferredLocale: str
      Represents the end-user's preferred language tag.
    preferredZoneinfo: str
      Represents the end-user's preferred time zone.
    roles: list[str]
      List of system roles to which the user has been assigned. Only returned when permitted by access control. Deprecated. Use `assignedRoles` instead.
    status: Literal["active", "invited", "disabled", "deleted"]
      The status of the user within the tenant.
    subject: str
      The unique user identitier from an identity provider.
    tenantId: str
      The tenant that the user belongs too.
    zoneinfo: str
      Represents the end-user's time zone.
    """

    assignedGroups: list[UserAssignedGroups] = None
    assignedRoles: list[UserAssignedRoles] = None
    created: str = None
    createdAt: str = None
    email: str = None
    id: str = None
    inviteExpiry: float = None
    lastUpdated: str = None
    lastUpdatedAt: str = None
    links: UserLinks = None
    locale: str = None
    name: str = None
    picture: str = None
    preferredLocale: str = None
    preferredZoneinfo: str = None
    roles: list[str] = None
    status: Literal["active", "invited", "disabled", "deleted"] = None
    subject: str = None
    tenantId: str = None
    zoneinfo: str = None

    def __init__(self_, **kvargs):
        if "assignedGroups" in kvargs and kvargs["assignedGroups"] is not None:
            if all(
                f"list[{type(e).__name__}]" == User.__annotations__["assignedGroups"]
                for e in kvargs["assignedGroups"]
            ):
                self_.assignedGroups = kvargs["assignedGroups"]
            else:
                self_.assignedGroups = [
                    UserAssignedGroups(**e) for e in kvargs["assignedGroups"]
                ]
        if "assignedRoles" in kvargs and kvargs["assignedRoles"] is not None:
            if all(
                f"list[{type(e).__name__}]" == User.__annotations__["assignedRoles"]
                for e in kvargs["assignedRoles"]
            ):
                self_.assignedRoles = kvargs["assignedRoles"]
            else:
                self_.assignedRoles = [
                    UserAssignedRoles(**e) for e in kvargs["assignedRoles"]
                ]
        if "created" in kvargs and kvargs["created"] is not None:
            self_.created = kvargs["created"]
        if "createdAt" in kvargs and kvargs["createdAt"] is not None:
            self_.createdAt = kvargs["createdAt"]
        if "email" in kvargs and kvargs["email"] is not None:
            self_.email = kvargs["email"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "inviteExpiry" in kvargs and kvargs["inviteExpiry"] is not None:
            self_.inviteExpiry = kvargs["inviteExpiry"]
        if "lastUpdated" in kvargs and kvargs["lastUpdated"] is not None:
            self_.lastUpdated = kvargs["lastUpdated"]
        if "lastUpdatedAt" in kvargs and kvargs["lastUpdatedAt"] is not None:
            self_.lastUpdatedAt = kvargs["lastUpdatedAt"]
        if "links" in kvargs and kvargs["links"] is not None:
            if type(kvargs["links"]).__name__ == User.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = UserLinks(**kvargs["links"])
        if "locale" in kvargs and kvargs["locale"] is not None:
            self_.locale = kvargs["locale"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "picture" in kvargs and kvargs["picture"] is not None:
            self_.picture = kvargs["picture"]
        if "preferredLocale" in kvargs and kvargs["preferredLocale"] is not None:
            self_.preferredLocale = kvargs["preferredLocale"]
        if "preferredZoneinfo" in kvargs and kvargs["preferredZoneinfo"] is not None:
            self_.preferredZoneinfo = kvargs["preferredZoneinfo"]
        if "roles" in kvargs and kvargs["roles"] is not None:
            self_.roles = kvargs["roles"]
        if "status" in kvargs and kvargs["status"] is not None:
            self_.status = kvargs["status"]
        if "subject" in kvargs and kvargs["subject"] is not None:
            self_.subject = kvargs["subject"]
        if "tenantId" in kvargs and kvargs["tenantId"] is not None:
            self_.tenantId = kvargs["tenantId"]
        if "zoneinfo" in kvargs and kvargs["zoneinfo"] is not None:
            self_.zoneinfo = kvargs["zoneinfo"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)

    def delete(self) -> None:
        """
        Delete user by ID
        Deletes the requested user.

        Parameters
        ----------
        """
        self.auth.rest(
            path="/users/{userId}".replace("{userId}", self.id),
            method="DELETE",
            params={},
            data=None,
        )

    def patch(self, data: JSONPatchArray) -> None:
        """
        Update user by ID
        Updates fields for a user resource

        Parameters
        ----------
        data: JSONPatchArray
          An array of JSON Patch documents
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        self.auth.rest(
            path="/users/{userId}".replace("{userId}", self.id),
            method="PATCH",
            params={},
            data=data,
        )


@dataclass
class Error:
    """
    An error object describing the error.

    Attributes
    ----------
    code: str
      The error code.
    detail: str
      A human-readable explanation specific to this occurrence of the problem.
    meta: object
      Additional properties relating to the error.
    source: ErrorSource
      References to the source of the error.
    status: float
      The HTTP status code.
    title: str
      Summary of the problem.
    """

    code: str = None
    detail: str = None
    meta: object = None
    source: ErrorSource = None
    status: float = None
    title: str = None

    def __init__(self_, **kvargs):
        if "code" in kvargs and kvargs["code"] is not None:
            self_.code = kvargs["code"]
        if "detail" in kvargs and kvargs["detail"] is not None:
            self_.detail = kvargs["detail"]
        if "meta" in kvargs and kvargs["meta"] is not None:
            self_.meta = kvargs["meta"]
        if "source" in kvargs and kvargs["source"] is not None:
            if type(kvargs["source"]).__name__ == Error.__annotations__["source"]:
                self_.source = kvargs["source"]
            else:
                self_.source = ErrorSource(**kvargs["source"])
        if "status" in kvargs and kvargs["status"] is not None:
            self_.status = kvargs["status"]
        if "title" in kvargs and kvargs["title"] is not None:
            self_.title = kvargs["title"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class ErrorSource:
    """
    References to the source of the error.

    Attributes
    ----------
    parameter: str
      The URI query parameter that caused the error.
    pointer: str
      A JSON Pointer to the property that caused the error.
    """

    parameter: str = None
    pointer: str = None

    def __init__(self_, **kvargs):
        if "parameter" in kvargs and kvargs["parameter"] is not None:
            self_.parameter = kvargs["parameter"]
        if "pointer" in kvargs and kvargs["pointer"] is not None:
            self_.pointer = kvargs["pointer"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Errors:
    """
    The error response object describing the error from the handling of an HTTP request.

    Attributes
    ----------
    errors: list[Error]
      An array of errors related to the operation.
    traceId: str
      A unique identifier for tracing the error.
    """

    errors: list[Error] = None
    traceId: str = None

    def __init__(self_, **kvargs):
        if "errors" in kvargs and kvargs["errors"] is not None:
            if all(
                f"list[{type(e).__name__}]" == Errors.__annotations__["errors"]
                for e in kvargs["errors"]
            ):
                self_.errors = kvargs["errors"]
            else:
                self_.errors = [Error(**e) for e in kvargs["errors"]]
        if "traceId" in kvargs and kvargs["traceId"] is not None:
            self_.traceId = kvargs["traceId"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Filter:
    """
    An advanced query filter to be used for complex user querying in the tenant.

    Attributes
    ----------
    filter: str
      The advanced filtering to be applied the query. All conditional statements within this query parameter are case insensitive.
    """

    filter: str = None

    def __init__(self_, **kvargs):
        if "filter" in kvargs and kvargs["filter"] is not None:
            self_.filter = kvargs["filter"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class InviteDataResponse:
    """
    Data list - ResultItem or ErrorItem for each InviteeItem.

    Attributes
    ----------
    data: list[any]
    """

    data: list[any] = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            self_.data = kvargs["data"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class InviteItem:
    """

    Attributes
    ----------
    email: str
      Email address for this invitee. Example - "foo@qlik.com".
    language: str
      Optional ISO 639-1 2 letter code for invite language. Defaults to 'en' when missing or not found.
    name: str
      Optional display name for this invitee. Example - "Elvis Presley".
    resend: bool
      Flag - when true invite message is sent to inactive or invited users. Typically used to force email resend to users who are not yet active.
    """

    email: str = None
    language: str = None
    name: str = None
    resend: bool = None

    def __init__(self_, **kvargs):
        if "email" in kvargs and kvargs["email"] is not None:
            self_.email = kvargs["email"]
        if "language" in kvargs and kvargs["language"] is not None:
            self_.language = kvargs["language"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "resend" in kvargs and kvargs["resend"] is not None:
            self_.resend = kvargs["resend"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class InviteRequest:
    """

    Attributes
    ----------
    invitees: list[InviteItem]
      List of invitees who should receive an invite email.
    """

    invitees: list[InviteItem] = None

    def __init__(self_, **kvargs):
        if "invitees" in kvargs and kvargs["invitees"] is not None:
            if all(
                f"list[{type(e).__name__}]" == InviteRequest.__annotations__["invitees"]
                for e in kvargs["invitees"]
            ):
                self_.invitees = kvargs["invitees"]
            else:
                self_.invitees = [InviteItem(**e) for e in kvargs["invitees"]]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class JSONPatch:
    """
    A JSON Patch document as defined in http://tools.ietf.org/html/rfc6902.

    Attributes
    ----------
    op: Literal["replace", "set (Deprecated)", "unset (Deprecated)", "add (Deprecated)", "renew"]
      The operation to be performed.
    path: Literal["/name", "/roles (Deprecated)", "/assignedRoles", "/inviteExpiry", "/preferredZoneinfo", "/preferredLocale", "/status"]
      A JSON Pointer.
    value: object
      The value to be used for this operation.
    """

    op: Literal[
        "replace", "set (Deprecated)", "unset (Deprecated)", "add (Deprecated)", "renew"
    ] = None
    path: Literal[
        "/name",
        "/roles (Deprecated)",
        "/assignedRoles",
        "/inviteExpiry",
        "/preferredZoneinfo",
        "/preferredLocale",
        "/status",
    ] = None
    value: str | bool | list[any] | list[JSONPatchAssignedRolesRefIDs] | list[
        any
    ] = None

    def __init__(self_, **kvargs):
        if "op" in kvargs and kvargs["op"] is not None:
            self_.op = kvargs["op"]
        if "path" in kvargs and kvargs["path"] is not None:
            self_.path = kvargs["path"]
        if "value" in kvargs and kvargs["value"] is not None:
            self_.value = kvargs["value"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class JSONPatchArray(List["JSONPatch"]):
    """
    An array of JSON Patch documents

    Attributes
    ----------
    """

    def __init__(self_, elements):
        for e in elements:
            self_.append(JSONPatch(**e))


@dataclass
class JSONPatchAssignedRolesRefIDs:
    """
    represents a role entity stored in the database

    Attributes
    ----------
    id: str
      The unique role identitier
    """

    id: str = None

    def __init__(self_, **kvargs):
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Metadata:
    """
    An object containing the metadata for the user configuration.

    Attributes
    ----------
    valid_roles: list[str]
      List of system roles to which the user can be assigned.
    """

    valid_roles: list[str] = None

    def __init__(self_, **kvargs):
        if "valid_roles" in kvargs and kvargs["valid_roles"] is not None:
            self_.valid_roles = kvargs["valid_roles"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserAssignedGroups:
    """
    represents a group entity stored in the database

    Attributes
    ----------
    assignedRoles: list[UserAssignedGroupsAssignedRoles]
      An array of role references.
    id: str
      The unique group identitier
    name: str
      The group name
    """

    assignedRoles: list[UserAssignedGroupsAssignedRoles] = None
    id: str = None
    name: str = None

    def __init__(self_, **kvargs):
        if "assignedRoles" in kvargs and kvargs["assignedRoles"] is not None:
            if all(
                f"list[{type(e).__name__}]"
                == UserAssignedGroups.__annotations__["assignedRoles"]
                for e in kvargs["assignedRoles"]
            ):
                self_.assignedRoles = kvargs["assignedRoles"]
            else:
                self_.assignedRoles = [
                    UserAssignedGroupsAssignedRoles(**e)
                    for e in kvargs["assignedRoles"]
                ]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserAssignedGroupsAssignedRoles:
    """
    represents a role entity stored in the database

    Attributes
    ----------
    id: str
      The unique role identitier
    level: Literal["admin", "user"]
      The role level
    name: str
      The role name
    permissions: list[str]
      An array of permissions associated to a given role.
    type: Literal["default"]
      The type of role
    """

    id: str = None
    level: Literal["admin", "user"] = None
    name: str = None
    permissions: list[str] = None
    type: Literal["default"] = None

    def __init__(self_, **kvargs):
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "level" in kvargs and kvargs["level"] is not None:
            self_.level = kvargs["level"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "permissions" in kvargs and kvargs["permissions"] is not None:
            self_.permissions = kvargs["permissions"]
        if "type" in kvargs and kvargs["type"] is not None:
            self_.type = kvargs["type"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserAssignedRoles:
    """
    represents a role entity stored in the database

    Attributes
    ----------
    id: str
      The unique role identitier
    level: Literal["admin", "user"]
      The role level
    name: str
      The role name
    permissions: list[str]
      An array of permissions associated to a given role.
    type: Literal["default"]
      The type of role
    """

    id: str = None
    level: Literal["admin", "user"] = None
    name: str = None
    permissions: list[str] = None
    type: Literal["default"] = None

    def __init__(self_, **kvargs):
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "level" in kvargs and kvargs["level"] is not None:
            self_.level = kvargs["level"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "permissions" in kvargs and kvargs["permissions"] is not None:
            self_.permissions = kvargs["permissions"]
        if "type" in kvargs and kvargs["type"] is not None:
            self_.type = kvargs["type"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserCount:
    """
    The result object for the user count.

    Attributes
    ----------
    total: float
      The total number of users in the tenant.
    """

    total: float = None

    def __init__(self_, **kvargs):
        if "total" in kvargs and kvargs["total"] is not None:
            self_.total = kvargs["total"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserLinks:
    """
    Pagination links to the user.

    Attributes
    ----------
    self: UserLinksSelf
      A link to this user.
    """

    self: UserLinksSelf = None

    def __init__(self_, **kvargs):
        if "self" in kvargs and kvargs["self"] is not None:
            if type(kvargs["self"]).__name__ == UserLinks.__annotations__["self"]:
                self_.self = kvargs["self"]
            else:
                self_.self = UserLinksSelf(**kvargs["self"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserLinksSelf:
    """
    A link to this user.

    Attributes
    ----------
    href: str
      URL that defines the resource.
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserPostSchema:
    """

    Attributes
    ----------
    assignedRoles: object
      The roles to assign to the user.
    email: str
      The email address for the user. This is a required field when inviting a user.
    name: str
      The name of the user.
    picture: str
      A static url linking to the avatar of the user.
    roles: list[str]
      List of system roles to which the user has been assigned. Only returned when permitted by access control.
    status: Literal["invited"]
      The status of the created user within the tenant.
    subject: str
      The unique user identitier from an identity provider.
    tenantId: str
      The tenant that the user will belong too.
    """

    assignedRoles: list[UserPostSchemaAssignedRolesRefIDs] | list[any] = None
    email: str = None
    name: str = None
    picture: str = None
    roles: list[str] = None
    status: Literal["invited"] = None
    subject: str = None
    tenantId: str = None

    def __init__(self_, **kvargs):
        if "assignedRoles" in kvargs and kvargs["assignedRoles"] is not None:
            self_.assignedRoles = kvargs["assignedRoles"]
        if "email" in kvargs and kvargs["email"] is not None:
            self_.email = kvargs["email"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "picture" in kvargs and kvargs["picture"] is not None:
            self_.picture = kvargs["picture"]
        if "roles" in kvargs and kvargs["roles"] is not None:
            self_.roles = kvargs["roles"]
        if "status" in kvargs and kvargs["status"] is not None:
            self_.status = kvargs["status"]
        if "subject" in kvargs and kvargs["subject"] is not None:
            self_.subject = kvargs["subject"]
        if "tenantId" in kvargs and kvargs["tenantId"] is not None:
            self_.tenantId = kvargs["tenantId"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UserPostSchemaAssignedRolesRefIDs:
    """
    represents a role entity stored in the database

    Attributes
    ----------
    id: str
      The unique role identitier
    """

    id: str = None

    def __init__(self_, **kvargs):
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UsersClass:
    """

    Attributes
    ----------
    data: list[User]
      List of users.
    links: UsersLinks
      Pagination links
    """

    data: list[User] = None
    links: UsersLinks = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]" == UsersClass.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [User(**e) for e in kvargs["data"]]
        if "links" in kvargs and kvargs["links"] is not None:
            if type(kvargs["links"]).__name__ == UsersClass.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = UsersLinks(**kvargs["links"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UsersLinks:
    """
    Pagination links

    Attributes
    ----------
    next: UsersLinksNext
      Link information for next page
    prev: UsersLinksPrev
      Link information for previous page
    self: UsersLinksSelf
      Link information for current page
    """

    next: UsersLinksNext = None
    prev: UsersLinksPrev = None
    self: UsersLinksSelf = None

    def __init__(self_, **kvargs):
        if "next" in kvargs and kvargs["next"] is not None:
            if type(kvargs["next"]).__name__ == UsersLinks.__annotations__["next"]:
                self_.next = kvargs["next"]
            else:
                self_.next = UsersLinksNext(**kvargs["next"])
        if "prev" in kvargs and kvargs["prev"] is not None:
            if type(kvargs["prev"]).__name__ == UsersLinks.__annotations__["prev"]:
                self_.prev = kvargs["prev"]
            else:
                self_.prev = UsersLinksPrev(**kvargs["prev"])
        if "self" in kvargs and kvargs["self"] is not None:
            if type(kvargs["self"]).__name__ == UsersLinks.__annotations__["self"]:
                self_.self = kvargs["self"]
            else:
                self_.self = UsersLinksSelf(**kvargs["self"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UsersLinksNext:
    """
    Link information for next page

    Attributes
    ----------
    href: str
      URL to the next page of records
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UsersLinksPrev:
    """
    Link information for previous page

    Attributes
    ----------
    href: str
      URL to the previous page of records
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UsersLinksSelf:
    """
    Link information for current page

    Attributes
    ----------
    href: str
      URL to the current page of records
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Users:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def count(self, tenantId: str = None) -> UserCount:
        """
        Count users
        Returns the number of users in a given tenant

        Parameters
        ----------
        tenantId: str = None
          The tenant ID to filter by.
        """
        query_params = {}
        if tenantId is not None:
            query_params["tenantId"] = tenantId
            warnings.warn("tenantId is deprecated", DeprecationWarning, stacklevel=2)
        response = self.auth.rest(
            path="/users/actions/count",
            method="GET",
            params=query_params,
            data=None,
        )
        obj = UserCount(**response.json())
        obj.auth = self.auth
        return obj

    def filter(
        self,
        data: Filter = None,
        fields: str = None,
        limit: float = 20,
        next: str = None,
        prev: str = None,
        sort: Literal["name", "+name", "-name"] = "+name",
    ) -> ListableResource[User]:
        """
        Filter users
        Retrieves a list of users matching the filter using an advanced query string.

        Parameters
        ----------
        data: Filter = None
          An advanced query filter to be used for complex user querying in the tenant.
        fields: str = None
          A comma-delimited string of the requested fields per entity. If the 'links' value is omitted, then the entity HATEOAS link will also be omitted.
        limit: float = 20
          The number of user entries to retrieve.
        next: str = None
          Get users with IDs that are higher than the target user ID. Cannot be used in conjunction with prev.
        prev: str = None
          Get users with IDs that are lower than the target user ID. Cannot be used in conjunction with next.
        sort: Literal["name", "+name", "-name"] = "+name"
          The field to sort by, with +/- prefix indicating sort order
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        query_params = {}
        if fields is not None:
            query_params["fields"] = fields
        if limit is not None:
            query_params["limit"] = limit
        if next is not None:
            query_params["next"] = next
        if prev is not None:
            query_params["prev"] = prev
        if sort is not None:
            query_params["sort"] = sort
        response = self.auth.rest(
            path="/users/actions/filter",
            method="POST",
            params=query_params,
            data=data,
        )
        return ListableResource(
            response=response.json(),
            cls=User,
            auth=self.auth,
            path="/users/actions/filter",
            query_params=query_params,
        )

    def invite(self, data: InviteRequest) -> InviteDataResponse:
        """
        Invite one or more users by email address.

        Parameters
        ----------
        data: InviteRequest
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/users/actions/invite",
            method="POST",
            params={},
            data=data,
        )
        obj = InviteDataResponse(**response.json())
        obj.auth = self.auth
        return obj

    def get_me(self) -> User:
        """
        Get my user
        Redirects to retrieve the user resource associated with the JWT claims.

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/users/me",
            method="GET",
            params={},
            data=None,
        )
        obj = User(**response.json())
        obj.auth = self.auth
        return obj

    def get_metadata(self) -> Metadata:
        """
        Get configuration metadata
        Returns the metadata with regard to the user configuration. Deprecated, use GET /v1/roles instead.

        Parameters
        ----------
        """
        warnings.warn("get_metadata is deprecated", DeprecationWarning, stacklevel=2)
        response = self.auth.rest(
            path="/users/metadata",
            method="GET",
            params={},
            data=None,
        )
        obj = Metadata(**response.json())
        obj.auth = self.auth
        return obj

    def get(self, userId: str, fields: str = None) -> User:
        """
        Get user by ID
        Returns the requested user.

        Parameters
        ----------
        userId: str
          The user's unique identifier
        fields: str = None
          A comma-delimited string of the requested fields per entity. If the 'links' value is omitted, then the entity HATEOAS link will also be omitted.
        """
        query_params = {}
        if fields is not None:
            query_params["fields"] = fields
        response = self.auth.rest(
            path="/users/{userId}".replace("{userId}", userId),
            method="GET",
            params=query_params,
            data=None,
        )
        obj = User(**response.json())
        obj.auth = self.auth
        return obj

    def get_users(
        self,
        fields: str = None,
        filter: str = None,
        limit: float = 20,
        next: str = None,
        prev: str = None,
        sort: Literal["name", "+name", "-name"] = "+name",
        totalResults: bool = None,
        email: str = None,
        endingBefore: str = None,
        role: str = None,
        sortBy: Literal["name"] = "name",
        sortOrder: Literal["asc", "desc"] = "asc",
        startingAfter: str = None,
        status: Literal["active", "invited", "disabled", "deleted"] = "active",
        subject: str = None,
        tenantId: str = None,
    ) -> ListableResource[User]:
        """
        List users
        Returns a list of users using cursor-based pagination.

        Parameters
        ----------
        fields: str = None
          A comma-delimited string of the requested fields per entity. If the 'links' value is omitted, then the entity HATEOAS link will also be omitted.
        filter: str = None
          The advanced filtering to use for the query. Refer to RFC 7644 https://datatracker.ietf.org/doc/rfc7644/ for the syntax. Cannot be combined with any of the fields marked as deprecated. All conditional statements within this query parameter are case insensitive.

          The following fields support the `eq` operator: `id`, `subject`, `name`, `email`, `status`, `clientId`, `assignedRoles.id` `assignedRoles.name`, `assignedGroups.id`, `assignedGroupsAssignedRoles.name`

          Additionally, the following fields support the `co` operator: `name`, `email`, `subject`

          Queries may be rate limited if they differ greatly from these examples:

          ```
          (id eq "62716ab404a7bd8626af9bd6" or id eq "62716ac4c7e500e13ff5fa22") and (status eq "active" or status eq "disabled")
          ```

          ```
          name co "query" or email co "query" or subject co "query" or id eq "query" or assignedRoles.name eq "query"
          ```

          Any filters for status must be grouped together and applied to the whole query.

          Valid:

          ```
          (name eq "Bob" or name eq "Alice") and (status eq "active" or status eq "disabled")
          ```

          Invalid:

          ```
          name eq "Bob" or name eq "Alice" and (status eq "active" or status eq "disabled")
          ```
        limit: float = 20
          The number of user entries to retrieve.
        next: str = None
          Get users that come after this cursor value when sorted. Cannot be used in conjunction with `prev`.
        prev: str = None
          Get users that come before this cursor value when sorted. Cannot be used in conjunction with `next`.
        sort: Literal["name", "+name", "-name"] = "+name"
          The field to sort by, with +/- prefix indicating sort order
        totalResults: bool = None
          Whether to return a total match count in the result. Defaults to false. It will trigger an extra DB query to count, reducing the efficiency of the endpoint.
        email: str = None
          The email to filter by. Deprecated. Use the new `filter` parameter to provide an advanced query filter.
        endingBefore: str = None
          Get users with IDs that are lower than the target user ID. Cannot be used in conjunction with startingAfter. Deprecated. Use `prev` instead.
        role: str = None
          The role to filter by. Deprecated.
        sortBy: Literal["name"] = "name"
          The user parameter to sort by. Deprecated. Use `sort` instead.
        sortOrder: Literal["asc", "desc"] = "asc"
          The sort order, either ascending or descending. Deprecated. Use `sort` instead.
        startingAfter: str = None
          Get users with IDs that are higher than the target user ID. Cannot be used in conjunction with endingBefore. Deprecated. Use `next` instead.
        status: Literal["active", "invited", "disabled", "deleted"] = "active"
          The status to filter by. Supports multiple values delimited by commas. Deprecated. Use the new `filter` parameter to provide an advanced query filter.
        subject: str = None
          The subject to filter by. Deprecated. Use the new `filter` parameter to provide an advanced query filter.
        tenantId: str = None
          The tenant ID to filter by. Deprecated.
        """
        query_params = {}
        if fields is not None:
            query_params["fields"] = fields
        if filter is not None:
            query_params["filter"] = filter
        if limit is not None:
            query_params["limit"] = limit
        if next is not None:
            query_params["next"] = next
        if prev is not None:
            query_params["prev"] = prev
        if sort is not None:
            query_params["sort"] = sort
        if totalResults is not None:
            query_params["totalResults"] = totalResults
        if email is not None:
            query_params["email"] = email
            warnings.warn("email is deprecated", DeprecationWarning, stacklevel=2)
        if endingBefore is not None:
            query_params["endingBefore"] = endingBefore
            warnings.warn(
                "endingBefore is deprecated", DeprecationWarning, stacklevel=2
            )
        if role is not None:
            query_params["role"] = role
            warnings.warn("role is deprecated", DeprecationWarning, stacklevel=2)
        if sortBy is not None:
            query_params["sortBy"] = sortBy
            warnings.warn("sortBy is deprecated", DeprecationWarning, stacklevel=2)
        if sortOrder is not None:
            query_params["sortOrder"] = sortOrder
            warnings.warn("sortOrder is deprecated", DeprecationWarning, stacklevel=2)
        if startingAfter is not None:
            query_params["startingAfter"] = startingAfter
            warnings.warn(
                "startingAfter is deprecated", DeprecationWarning, stacklevel=2
            )
        if status is not None:
            query_params["status"] = status
            warnings.warn("status is deprecated", DeprecationWarning, stacklevel=2)
        if subject is not None:
            query_params["subject"] = subject
            warnings.warn("subject is deprecated", DeprecationWarning, stacklevel=2)
        if tenantId is not None:
            query_params["tenantId"] = tenantId
            warnings.warn("tenantId is deprecated", DeprecationWarning, stacklevel=2)
        response = self.auth.rest(
            path="/users",
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=User,
            auth=self.auth,
            path="/users",
            query_params=query_params,
        )

    def create(self, data: UserPostSchema) -> User:
        """
        Create user
        Creates an invited user.

        Parameters
        ----------
        data: UserPostSchema
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/users",
            method="POST",
            params={},
            data=data,
        )
        obj = User(**response.json())
        obj.auth = self.auth
        return obj



# This is spectacularly generated code by spectacular based on
# Qlik Cloud Services APIs

from __future__ import annotations

from dataclasses import dataclass
from typing import Literal

from ..auth import Auth, Config
from ..listable import ListableResource


@dataclass
class Role:
    """

    Attributes
    ----------
    createdAt: str
      The timestamp for when the role was created.
    description: str
      Descriptive text for the role.
    id: str
      The unique identifier for the role.
    lastUpdatedAt: str
      The timestamp for when the role was last updated.
    level: Literal["admin", "user"]
      The level of access associated to the role.
    links: RoleLinks
      Contains links for the role.
    name: str
      The name of the role.
    permissions: list[str]
      An array of permissions associated with the role.
    tenantId: str
      The tenant unique identifier associated with the given Role.
    type: Literal["default"]
      The type of role.
    """

    createdAt: str = None
    description: str = None
    id: str = None
    lastUpdatedAt: str = None
    level: Literal["admin", "user"] = None
    links: RoleLinks = None
    name: str = None
    permissions: list[str] = None
    tenantId: str = None
    type: Literal["default"] = None

    def __init__(self_, **kvargs):
        if "createdAt" in kvargs and kvargs["createdAt"] is not None:
            self_.createdAt = kvargs["createdAt"]
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "lastUpdatedAt" in kvargs and kvargs["lastUpdatedAt"] is not None:
            self_.lastUpdatedAt = kvargs["lastUpdatedAt"]
        if "level" in kvargs and kvargs["level"] is not None:
            self_.level = kvargs["level"]
        if "links" in kvargs and kvargs["links"] is not None:
            if type(kvargs["links"]).__name__ == Role.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = RoleLinks(**kvargs["links"])
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "permissions" in kvargs and kvargs["permissions"] is not None:
            self_.permissions = kvargs["permissions"]
        if "tenantId" in kvargs and kvargs["tenantId"] is not None:
            self_.tenantId = kvargs["tenantId"]
        if "type" in kvargs and kvargs["type"] is not None:
            self_.type = kvargs["type"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Links:
    """
    Contains pagination links

    Attributes
    ----------
    next: LinksNext
      Link to the next page of items
    prev: LinksPrev
      Link to the previous page of items
    self: LinksSelf
      Link to the current page of items
    """

    next: LinksNext = None
    prev: LinksPrev = None
    self: LinksSelf = None

    def __init__(self_, **kvargs):
        if "next" in kvargs and kvargs["next"] is not None:
            if type(kvargs["next"]).__name__ == Links.__annotations__["next"]:
                self_.next = kvargs["next"]
            else:
                self_.next = LinksNext(**kvargs["next"])
        if "prev" in kvargs and kvargs["prev"] is not None:
            if type(kvargs["prev"]).__name__ == Links.__annotations__["prev"]:
                self_.prev = kvargs["prev"]
            else:
                self_.prev = LinksPrev(**kvargs["prev"])
        if "self" in kvargs and kvargs["self"] is not None:
            if type(kvargs["self"]).__name__ == Links.__annotations__["self"]:
                self_.self = kvargs["self"]
            else:
                self_.self = LinksSelf(**kvargs["self"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class LinksNext:
    """
    Link to the next page of items

    Attributes
    ----------
    href: str
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class LinksPrev:
    """
    Link to the previous page of items

    Attributes
    ----------
    href: str
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class LinksSelf:
    """
    Link to the current page of items

    Attributes
    ----------
    href: str
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class ListRolesResult:
    """

    Attributes
    ----------
    data: list[Role]
      An array of roles.
    links: Links
      Contains pagination links
    totalResults: int
      Indicates the total number of matching documents. Will only be returned if the query parameter "totalResults" is true.
    """

    data: list[Role] = None
    links: Links = None
    totalResults: int = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]" == ListRolesResult.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [Role(**e) for e in kvargs["data"]]
        if "links" in kvargs and kvargs["links"] is not None:
            if (
                type(kvargs["links"]).__name__
                == ListRolesResult.__annotations__["links"]
            ):
                self_.links = kvargs["links"]
            else:
                self_.links = Links(**kvargs["links"])
        if "totalResults" in kvargs and kvargs["totalResults"] is not None:
            self_.totalResults = kvargs["totalResults"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class RoleLinks:
    """
    Contains links for the role.

    Attributes
    ----------
    self: RoleLinksSelf
    """

    self: RoleLinksSelf = None

    def __init__(self_, **kvargs):
        if "self" in kvargs and kvargs["self"] is not None:
            if type(kvargs["self"]).__name__ == RoleLinks.__annotations__["self"]:
                self_.self = kvargs["self"]
            else:
                self_.self = RoleLinksSelf(**kvargs["self"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class RoleLinksSelf:
    """

    Attributes
    ----------
    href: str
      Link to the role.
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Roles:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def get(self, id: str) -> Role:
        """
        Get role by ID
        Returns the requested role.

        Parameters
        ----------
        id: str
          The role's unique identifier
        """
        response = self.auth.rest(
            path="/roles/{id}".replace("{id}", id),
            method="GET",
            params={},
            data=None,
        )
        obj = Role(**response.json())
        obj.auth = self.auth
        return obj

    def get_roles(
        self,
        filter: str = None,
        limit: float = 20,
        next: str = None,
        prev: str = None,
        sort: str = None,
        totalResults: bool = None,
    ) -> ListableResource[Role]:
        """
        List roles
        Returns a list of roles using cursor-based pagination.

        Parameters
        ----------
        filter: str = None
          The advanced filtering to use for the query. Refer to RFC 7644 https://datatracker.ietf.org/doc/rfc7644/ for the syntax. All conditional statements within this query parameter are case insensitive.
        limit: float = 20
          The number of roles to retrieve.
        next: str = None
          The next page cursor.
        prev: str = None
          The previous page cursor.
        sort: str = None
          Optional resource field name to sort on, eg. name. Can be prefixed with +/- to determine order, defaults to (+) ascending.
        totalResults: bool = None
          Determines wether to return a count of the total records matched in the query. Defaults to false.
        """
        query_params = {}
        if filter is not None:
            query_params["filter"] = filter
        if limit is not None:
            query_params["limit"] = limit
        if next is not None:
            query_params["next"] = next
        if prev is not None:
            query_params["prev"] = prev
        if sort is not None:
            query_params["sort"] = sort
        if totalResults is not None:
            query_params["totalResults"] = totalResults
        response = self.auth.rest(
            path="/roles",
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=Role,
            auth=self.auth,
            path="/roles",
            query_params=query_params,
        )



import random
import string
import time
from typing import List

import jwt


def _generate_id(length: int) -> str:
    return "".join(random.choices(string.ascii_letters + string.digits, k=length))


def generate_signed_token(
    crt: str,
    sub: str,
    sub_type: str,
    name: str,
    email: str,
    expires_in: int,
    not_before: int,
    issuer: str,
    keyid: str,
    email_verified: bool = True,
    jti: str = None,
    groups: List[str] = [],
    algorithm: str = "RS256",
    audience: str = "qlik.api/login/jwt-session",
) -> str:
    """
    helper method for generating signed tokens for exchange with the jwt-session auth flow

    Parameters
    ----------
    crt: str
        string containing the private key certificate (ex: PEM formated private key)
    sub: str
        main identifier (aka subject) of the user.
    sub_type: str
        type of identifier the sub represents. In this case, user is the only applicable value.
    name: str
        friendly name to apply to the user.
    email: str
        email address of the user.
    email_verified: bool, default True
        claim indicating that the JWT source has verified that the email address belongs to the subject.
    expires_in: str
        the lifespan of the resulting JWT.
    not_before: str
        time before which the JWT MUST NOT be accepted for processing.
    issuer: str
        value created or supplied previously with identity provider configuration.
    keyid: str
        value created or supplied previously with identity provider configuration.
    jti: str = None
        JWT ID claim provides a unique identifier for the JWT. Needs to be unique
    groups: list[str]
    algorithm: str, default "RS256"
        The algorithm must be set to "RS256"
    audience: str, default "qlik.api/login/jwt-session",
        audience must be set to "qlik.api/login/jwt-session"

    Notes
    -----
    For more information about the JWT-session auth flow,
    please check https://qlik.dev/tutorials/create-signed-tokens-for-jwt-authorization

    Example
    --------
    >>> with open('private_key.pem') as f:
    ...   private_key = f.read()
    ...   signed_token = generate_signed_token(
    ...       crt=private_key,
    ...       sub="test-user-id",
    ...       sub_type="user",
    ...       name="Hardcore Harry",
    ...       email="harry@example.com",
    ...       email_verified=True,
    ...       groups=["Administrators", "HardCoreGroup"],
    ...       expires_in=datetime.datetime.now() + datetime.timedelta(minutes=30),
    ...       not_before=0,
    ...       keyid="platform-sdks-jwt",
    ...       issuer="https://platform-sdks.qlik.dev",
    ...   )
    """
    if jti is None:
        jti = _generate_id(8)
    claims = {
        "jti": jti,
        "sub": sub,
        "subType": sub_type,
        "name": name,
        "email": email,
        "email_verified": email_verified,
        "groups": groups,
        "exp": expires_in,
        "nbf": not_before,
        "iat": int(time.time()),
        "iss": issuer,
        "aud": audience,
    }
    singning_options = {"kid": keyid}
    token = jwt.encode(
        payload=claims, key=crt, algorithm=algorithm, headers=singning_options
    )
    return token



# This is spectacularly generated code by spectacular based on
# Qlik Cloud Services APIs

from __future__ import annotations

from dataclasses import asdict, dataclass
from typing import Literal

from ..auth import Auth, Config
from ..listable import ListableResource


@dataclass
class AutomationDetailResponseObject:
    """

    Attributes
    ----------
    connectorIds: list[str]
      A list of connectors used inside the workspace.
    createdAt: str
    description: str
    endpointIds: list[str]
      A list of endpoints used inside the workspace.
    executionToken: str
    id: str
    lastRun: RunDetailResponseObject
    lastRunAt: str
    lastRunStatus: Literal["failed", "finished", "finished with warnings", "must stop", "not started", "paused", "running", "starting", "stopped"]
    name: str
    ownerId: str
    runMode: Literal["manual", "scheduled", "triggered", "webhook"]
    schedules: list[ScheduleResponseObject]
    snippetIds: list[str]
      A list of snippets used inside the workspace.
    state: Literal["available", "unavailable", "disabled"]
    updatedAt: str
    workspace: object
      The workspace generated by the Qlik Application Automations editor.
    """

    connectorIds: list[str] = None
    createdAt: str = None
    description: str = None
    endpointIds: list[str] = None
    executionToken: str = None
    id: str = None
    lastRun: RunDetailResponseObject = None
    lastRunAt: str = None
    lastRunStatus: Literal[
        "failed",
        "finished",
        "finished with warnings",
        "must stop",
        "not started",
        "paused",
        "running",
        "starting",
        "stopped",
    ] = None
    name: str = None
    ownerId: str = None
    runMode: Literal["manual", "scheduled", "triggered", "webhook"] = None
    schedules: list[ScheduleResponseObject] = None
    snippetIds: list[str] = None
    state: Literal["available", "unavailable", "disabled"] = "available"
    updatedAt: str = None
    workspace: object = None

    def __init__(self_, **kvargs):
        if "connectorIds" in kvargs and kvargs["connectorIds"] is not None:
            self_.connectorIds = kvargs["connectorIds"]
        if "createdAt" in kvargs and kvargs["createdAt"] is not None:
            self_.createdAt = kvargs["createdAt"]
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "endpointIds" in kvargs and kvargs["endpointIds"] is not None:
            self_.endpointIds = kvargs["endpointIds"]
        if "executionToken" in kvargs and kvargs["executionToken"] is not None:
            self_.executionToken = kvargs["executionToken"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "lastRun" in kvargs and kvargs["lastRun"] is not None:
            if (
                type(kvargs["lastRun"]).__name__
                == AutomationDetailResponseObject.__annotations__["lastRun"]
            ):
                self_.lastRun = kvargs["lastRun"]
            else:
                self_.lastRun = RunDetailResponseObject(**kvargs["lastRun"])
        if "lastRunAt" in kvargs and kvargs["lastRunAt"] is not None:
            self_.lastRunAt = kvargs["lastRunAt"]
        if "lastRunStatus" in kvargs and kvargs["lastRunStatus"] is not None:
            self_.lastRunStatus = kvargs["lastRunStatus"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "ownerId" in kvargs and kvargs["ownerId"] is not None:
            self_.ownerId = kvargs["ownerId"]
        if "runMode" in kvargs and kvargs["runMode"] is not None:
            self_.runMode = kvargs["runMode"]
        if "schedules" in kvargs and kvargs["schedules"] is not None:
            if all(
                f"list[{type(e).__name__}]"
                == AutomationDetailResponseObject.__annotations__["schedules"]
                for e in kvargs["schedules"]
            ):
                self_.schedules = kvargs["schedules"]
            else:
                self_.schedules = [
                    ScheduleResponseObject(**e) for e in kvargs["schedules"]
                ]
        if "snippetIds" in kvargs and kvargs["snippetIds"] is not None:
            self_.snippetIds = kvargs["snippetIds"]
        if "state" in kvargs and kvargs["state"] is not None:
            self_.state = kvargs["state"]
        if "updatedAt" in kvargs and kvargs["updatedAt"] is not None:
            self_.updatedAt = kvargs["updatedAt"]
        if "workspace" in kvargs and kvargs["workspace"] is not None:
            self_.workspace = kvargs["workspace"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)

    def copy(self, name: str = None) -> object:
        """
        Copy an automation.

        Parameters
        ----------
        name: str = None
          Name of the new automation.
        """
        data = {"name": name}
        response = self.auth.rest(
            path="/automations/{id}/actions/copy".replace("{id}", self.id),
            method="POST",
            params={},
            data=data,
        )
        return response.json()

    def disable(self) -> None:
        """
        Disables an automation.

        Parameters
        ----------
        """
        self.auth.rest(
            path="/automations/{id}/actions/disable".replace("{id}", self.id),
            method="POST",
            params={},
            data=None,
        )

    def enable(self) -> None:
        """
        Enable an automation.

        Parameters
        ----------
        """
        self.auth.rest(
            path="/automations/{id}/actions/enable".replace("{id}", self.id),
            method="POST",
            params={},
            data=None,
        )

    def move(self, userId: str = None) -> None:
        """
        Move an automation to a different user.
        This step will remove the history and change logs of this automation. All linked connections used inside an automation will be detached.

        Parameters
        ----------
        userId: str = None
        """
        data = {"userId": userId}
        self.auth.rest(
            path="/automations/{id}/actions/move".replace("{id}", self.id),
            method="POST",
            params={},
            data=data,
        )

    def export_run(self, runId: str) -> object:
        """
        Export a run.
        Creates a temporary pre-signed AWS S3 URL to download the raw logs of a completed run. This URL is valid for 5 minutes.

        Parameters
        ----------
        runId: str
          The unique identifier for the run.
        """
        response = self.auth.rest(
            path="/automations/{id}/runs/{runId}/actions/export".replace(
                "{id}", self.id
            ).replace("{runId}", runId),
            method="POST",
            params={},
            data=None,
        )
        return response.json()

    def retry_run(self, runId: str) -> None:
        """
        Retry a run.
        This action allows you to retry a specific run by creating a new run using the same inputs.

        Parameters
        ----------
        runId: str
          The unique identifier for the run.
        """
        self.auth.rest(
            path="/automations/{id}/runs/{runId}/actions/retry".replace(
                "{id}", self.id
            ).replace("{runId}", runId),
            method="POST",
            params={},
            data=None,
        )

    def stop_run(self, runId: str) -> None:
        """
        Stop a running automation.
        Force a run to be stopped immediately.

        Parameters
        ----------
        runId: str
          The unique identifier for the run.
        """
        self.auth.rest(
            path="/automations/{id}/runs/{runId}/actions/stop".replace(
                "{id}", self.id
            ).replace("{runId}", runId),
            method="POST",
            params={},
            data=None,
        )

    def get_run(self, runId: str) -> RunDetailResponseObject:
        """
        Retrieves a run

        Parameters
        ----------
        runId: str
          The unique identifier for the run.
        """
        response = self.auth.rest(
            path="/automations/{id}/runs/{runId}".replace("{id}", self.id).replace(
                "{runId}", runId
            ),
            method="GET",
            params={},
            data=None,
        )
        obj = RunDetailResponseObject(**response.json())
        obj.auth = self.auth
        return obj

    def get_runs(
        self,
        filter: str = None,
        limit: int = 10,
        sort: Literal[
            "id",
            "status",
            "startTime",
            "-id",
            "-status",
            "-startTime",
            "+id",
            "+status",
            "+startTime",
        ] = "id",
    ) -> ListableResource[RunListObject]:
        """
        Retrieves a list of runs for a specific automation.

        Parameters
        ----------
        filter: str = None
          Allowed filters: status, context, startTime and title
        limit: int = 10
          The number of runs to retrieve.
        sort: Literal["id", "status", "startTime", "-id", "-status", "-startTime", "+id", "+status", "+startTime"] = "id"
          The field to sort by, with +- prefix indicating sort order. (?query=-startTime => sort on the startTime field using descending order)
        """
        query_params = {}
        if filter is not None:
            query_params["filter"] = filter
        if limit is not None:
            query_params["limit"] = limit
        if sort is not None:
            query_params["sort"] = sort
        response = self.auth.rest(
            path="/automations/{id}/runs".replace("{id}", self.id),
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=RunListObject,
            auth=self.auth,
            path="/automations/{id}/runs".replace("{id}", self.id),
            query_params=query_params,
        )

    def create_run(self, data: RunDetailRequestObject) -> RunDetailResponseObject:
        """
        Create a run

        Parameters
        ----------
        data: RunDetailRequestObject
          Run object to create
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/automations/{id}/runs".replace("{id}", self.id),
            method="POST",
            params={},
            data=data,
        )
        obj = RunDetailResponseObject(**response.json())
        obj.auth = self.auth
        return obj

    def delete(self) -> None:
        """
        Delete an automation

        Parameters
        ----------
        """
        self.auth.rest(
            path="/automations/{id}".replace("{id}", self.id),
            method="DELETE",
            params={},
            data=None,
        )

    def set(
        self, data: AutomationDetailRequestObject
    ) -> AutomationDetailResponseObject:
        """
        Update an automation

        Parameters
        ----------
        data: AutomationDetailRequestObject
          Automation object to update
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/automations/{id}".replace("{id}", self.id),
            method="PUT",
            params={},
            data=data,
        )
        self.__init__(**response.json())
        return self


@dataclass
class AutomationDetailRequestObject:
    """

    Attributes
    ----------
    description: str
    name: str
    schedules: list[ScheduleRequestObject]
    state: Literal["available", "unavailable", "disabled"]
    workspace: object
      The workspace generated by the Qlik Application Automations editor.
    """

    description: str = None
    name: str = None
    schedules: list[ScheduleRequestObject] = None
    state: Literal["available", "unavailable", "disabled"] = "available"
    workspace: object = None

    def __init__(self_, **kvargs):
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "schedules" in kvargs and kvargs["schedules"] is not None:
            if all(
                f"list[{type(e).__name__}]"
                == AutomationDetailRequestObject.__annotations__["schedules"]
                for e in kvargs["schedules"]
            ):
                self_.schedules = kvargs["schedules"]
            else:
                self_.schedules = [
                    ScheduleRequestObject(**e) for e in kvargs["schedules"]
                ]
        if "state" in kvargs and kvargs["state"] is not None:
            self_.state = kvargs["state"]
        if "workspace" in kvargs and kvargs["workspace"] is not None:
            self_.workspace = kvargs["workspace"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class AutomationList:
    """

    Attributes
    ----------
    data: list[AutomationListObject]
    links: Links
    """

    data: list[AutomationListObject] = None
    links: Links = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]" == AutomationList.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [AutomationListObject(**e) for e in kvargs["data"]]
        if "links" in kvargs and kvargs["links"] is not None:
            if (
                type(kvargs["links"]).__name__
                == AutomationList.__annotations__["links"]
            ):
                self_.links = kvargs["links"]
            else:
                self_.links = Links(**kvargs["links"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class AutomationListObject:
    """

    Attributes
    ----------
    createdAt: str
    description: str
    id: str
    lastRunAt: str
    lastRunStatus: Literal["failed", "finished", "finished with warnings", "must stop", "not started", "paused", "running", "starting", "stopped"]
    name: str
    ownerId: str
    runMode: Literal["manual", "scheduled", "triggered", "webhook"]
    state: Literal["available", "unavailable", "disabled"]
    updatedAt: str
    """

    createdAt: str = None
    description: str = None
    id: str = None
    lastRunAt: str = None
    lastRunStatus: Literal[
        "failed",
        "finished",
        "finished with warnings",
        "must stop",
        "not started",
        "paused",
        "running",
        "starting",
        "stopped",
    ] = None
    name: str = None
    ownerId: str = None
    runMode: Literal["manual", "scheduled", "triggered", "webhook"] = None
    state: Literal["available", "unavailable", "disabled"] = "available"
    updatedAt: str = None

    def __init__(self_, **kvargs):
        if "createdAt" in kvargs and kvargs["createdAt"] is not None:
            self_.createdAt = kvargs["createdAt"]
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "lastRunAt" in kvargs and kvargs["lastRunAt"] is not None:
            self_.lastRunAt = kvargs["lastRunAt"]
        if "lastRunStatus" in kvargs and kvargs["lastRunStatus"] is not None:
            self_.lastRunStatus = kvargs["lastRunStatus"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "ownerId" in kvargs and kvargs["ownerId"] is not None:
            self_.ownerId = kvargs["ownerId"]
        if "runMode" in kvargs and kvargs["runMode"] is not None:
            self_.runMode = kvargs["runMode"]
        if "state" in kvargs and kvargs["state"] is not None:
            self_.state = kvargs["state"]
        if "updatedAt" in kvargs and kvargs["updatedAt"] is not None:
            self_.updatedAt = kvargs["updatedAt"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class AutomationUsageObject:
    """

    Attributes
    ----------
    guid: str
      The unique identifier for the automation.
    name: str
      The name for the automation.
    ownerId: str
      The unique identifier for the user who owns the automation
    """

    guid: str = None
    name: str = None
    ownerId: str = None

    def __init__(self_, **kvargs):
        if "guid" in kvargs and kvargs["guid"] is not None:
            self_.guid = kvargs["guid"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "ownerId" in kvargs and kvargs["ownerId"] is not None:
            self_.ownerId = kvargs["ownerId"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Links:
    """

    Attributes
    ----------
    next: PaginationLink
    prev: PaginationLink
    """

    next: PaginationLink = None
    prev: PaginationLink = None

    def __init__(self_, **kvargs):
        if "next" in kvargs and kvargs["next"] is not None:
            if type(kvargs["next"]).__name__ == Links.__annotations__["next"]:
                self_.next = kvargs["next"]
            else:
                self_.next = PaginationLink(**kvargs["next"])
        if "prev" in kvargs and kvargs["prev"] is not None:
            if type(kvargs["prev"]).__name__ == Links.__annotations__["prev"]:
                self_.prev = kvargs["prev"]
            else:
                self_.prev = PaginationLink(**kvargs["prev"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class PaginationLink:
    """

    Attributes
    ----------
    href: str
      The URL to a resource request
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class RunDetailRequestObject:
    """

    Attributes
    ----------
    context: Literal["api"]
      The source that triggers the automation will set the context.
    id: str
    inputs: object
    """

    context: Literal["api"] = None
    id: str = None
    inputs: object = None

    def __init__(self_, **kvargs):
        if "context" in kvargs and kvargs["context"] is not None:
            self_.context = kvargs["context"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "inputs" in kvargs and kvargs["inputs"] is not None:
            self_.inputs = kvargs["inputs"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class RunDetailResponseObject:
    """

    Attributes
    ----------
    context: Literal["test_run", "editor", "detail", "api_sync", "api_async", "webhook", "lookup"]
      The source that triggers the automation will set the context. Certain contexts impact the execution of an automation (for example, The "test_run" context will not process all results when listing items).
    createdAt: str
    error: object
    expectedInputs: object
    id: str
    inputs: object
    isArchived: bool
    isTestRun: bool
    metrics: RunDetailResponseObjectMetrics
    scheduledStartTime: str
    startTime: str
    status: Literal["failed", "finished", "finished with warnings", "must stop", "not started", "running", "starting", "stopped"]
    stopTime: str
    title: str
    updatedAt: str
    """

    context: Literal[
        "test_run", "editor", "detail", "api_sync", "api_async", "webhook", "lookup"
    ] = None
    createdAt: str = None
    error: object = None
    expectedInputs: object = None
    id: str = None
    inputs: object = None
    isArchived: bool = None
    isTestRun: bool = None
    metrics: RunDetailResponseObjectMetrics = None
    scheduledStartTime: str = None
    startTime: str = None
    status: Literal[
        "failed",
        "finished",
        "finished with warnings",
        "must stop",
        "not started",
        "running",
        "starting",
        "stopped",
    ] = None
    stopTime: str = None
    title: str = None
    updatedAt: str = None

    def __init__(self_, **kvargs):
        if "context" in kvargs and kvargs["context"] is not None:
            self_.context = kvargs["context"]
        if "createdAt" in kvargs and kvargs["createdAt"] is not None:
            self_.createdAt = kvargs["createdAt"]
        if "error" in kvargs and kvargs["error"] is not None:
            self_.error = kvargs["error"]
        if "expectedInputs" in kvargs and kvargs["expectedInputs"] is not None:
            self_.expectedInputs = kvargs["expectedInputs"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "inputs" in kvargs and kvargs["inputs"] is not None:
            self_.inputs = kvargs["inputs"]
        if "isArchived" in kvargs and kvargs["isArchived"] is not None:
            self_.isArchived = kvargs["isArchived"]
        if "isTestRun" in kvargs and kvargs["isTestRun"] is not None:
            self_.isTestRun = kvargs["isTestRun"]
        if "metrics" in kvargs and kvargs["metrics"] is not None:
            if (
                type(kvargs["metrics"]).__name__
                == RunDetailResponseObject.__annotations__["metrics"]
            ):
                self_.metrics = kvargs["metrics"]
            else:
                self_.metrics = RunDetailResponseObjectMetrics(**kvargs["metrics"])
        if "scheduledStartTime" in kvargs and kvargs["scheduledStartTime"] is not None:
            self_.scheduledStartTime = kvargs["scheduledStartTime"]
        if "startTime" in kvargs and kvargs["startTime"] is not None:
            self_.startTime = kvargs["startTime"]
        if "status" in kvargs and kvargs["status"] is not None:
            self_.status = kvargs["status"]
        if "stopTime" in kvargs and kvargs["stopTime"] is not None:
            self_.stopTime = kvargs["stopTime"]
        if "title" in kvargs and kvargs["title"] is not None:
            self_.title = kvargs["title"]
        if "updatedAt" in kvargs and kvargs["updatedAt"] is not None:
            self_.updatedAt = kvargs["updatedAt"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class RunDetailResponseObjectMetrics:
    """

    Attributes
    ----------
    blocks: list[RunDetailResponseObjectMetricsBlocks]
      List of blocks used during execution. Note: this list currently only contains endpointBlocks and snippetBlocks
    network: RunDetailResponseObjectMetricsNetwork
    totalApiCalls: int
      The number of API calls made.
    """

    blocks: list[RunDetailResponseObjectMetricsBlocks] = None
    network: RunDetailResponseObjectMetricsNetwork = None
    totalApiCalls: int = None

    def __init__(self_, **kvargs):
        if "blocks" in kvargs and kvargs["blocks"] is not None:
            if all(
                f"list[{type(e).__name__}]"
                == RunDetailResponseObjectMetrics.__annotations__["blocks"]
                for e in kvargs["blocks"]
            ):
                self_.blocks = kvargs["blocks"]
            else:
                self_.blocks = [
                    RunDetailResponseObjectMetricsBlocks(**e) for e in kvargs["blocks"]
                ]
        if "network" in kvargs and kvargs["network"] is not None:
            if (
                type(kvargs["network"]).__name__
                == RunDetailResponseObjectMetrics.__annotations__["network"]
            ):
                self_.network = kvargs["network"]
            else:
                self_.network = RunDetailResponseObjectMetricsNetwork(
                    **kvargs["network"]
                )
        if "totalApiCalls" in kvargs and kvargs["totalApiCalls"] is not None:
            self_.totalApiCalls = kvargs["totalApiCalls"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class RunDetailResponseObjectMetricsBlocks:
    """
    Information of the ended execution per block.

    Attributes
    ----------
    apiCalls: int
      API calls to external resources made by the current block.
    connectorId: str
      Unique identifier for the connector used within the block of the execution.
    endpointId: str
      Unique identifier for the snippet block used within the execution.
    rxBytes: int
      Total amount of received bytes sent by the current block.
    snippetId: str
      Unique identifier for the snippet block used within the execution.
    txBytes: int
      Total amount of sent bytes sent by the current block.
    type: Literal["snippetBlock", "endpointBlock"]
      Type of the block.
    """

    apiCalls: int = None
    connectorId: str = None
    endpointId: str = None
    rxBytes: int = None
    snippetId: str = None
    txBytes: int = None
    type: Literal["snippetBlock", "endpointBlock"] = None

    def __init__(self_, **kvargs):
        if "apiCalls" in kvargs and kvargs["apiCalls"] is not None:
            self_.apiCalls = kvargs["apiCalls"]
        if "connectorId" in kvargs and kvargs["connectorId"] is not None:
            self_.connectorId = kvargs["connectorId"]
        if "endpointId" in kvargs and kvargs["endpointId"] is not None:
            self_.endpointId = kvargs["endpointId"]
        if "rxBytes" in kvargs and kvargs["rxBytes"] is not None:
            self_.rxBytes = kvargs["rxBytes"]
        if "snippetId" in kvargs and kvargs["snippetId"] is not None:
            self_.snippetId = kvargs["snippetId"]
        if "txBytes" in kvargs and kvargs["txBytes"] is not None:
            self_.txBytes = kvargs["txBytes"]
        if "type" in kvargs and kvargs["type"] is not None:
            self_.type = kvargs["type"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class RunDetailResponseObjectMetricsNetwork:
    """

    Attributes
    ----------
    rxBytes: int
      The number of received bytes.
    txBytes: int
      The number of received bytes.
    """

    rxBytes: int = None
    txBytes: int = None

    def __init__(self_, **kvargs):
        if "rxBytes" in kvargs and kvargs["rxBytes"] is not None:
            self_.rxBytes = kvargs["rxBytes"]
        if "txBytes" in kvargs and kvargs["txBytes"] is not None:
            self_.txBytes = kvargs["txBytes"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class RunList:
    """

    Attributes
    ----------
    data: list[RunListObject]
    links: Links
    """

    data: list[RunListObject] = None
    links: Links = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]" == RunList.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [RunListObject(**e) for e in kvargs["data"]]
        if "links" in kvargs and kvargs["links"] is not None:
            if type(kvargs["links"]).__name__ == RunList.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = Links(**kvargs["links"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class RunListObject:
    """

    Attributes
    ----------
    context: Literal["test_run", "editor", "detail", "api_sync", "api_async", "webhook", "lookup"]
    createdAt: str
    error: object
    id: str
    isArchived: bool
    isTestRun: bool
    scheduledStartTime: str
    startTime: str
    status: Literal["failed", "finished", "finished with warnings", "must stop", "not started", "running", "starting", "stopped"]
    stopTime: str
    title: str
    updatedAt: str
    """

    context: Literal[
        "test_run", "editor", "detail", "api_sync", "api_async", "webhook", "lookup"
    ] = None
    createdAt: str = None
    error: object = None
    id: str = None
    isArchived: bool = None
    isTestRun: bool = None
    scheduledStartTime: str = None
    startTime: str = None
    status: Literal[
        "failed",
        "finished",
        "finished with warnings",
        "must stop",
        "not started",
        "running",
        "starting",
        "stopped",
    ] = None
    stopTime: str = None
    title: str = None
    updatedAt: str = None

    def __init__(self_, **kvargs):
        if "context" in kvargs and kvargs["context"] is not None:
            self_.context = kvargs["context"]
        if "createdAt" in kvargs and kvargs["createdAt"] is not None:
            self_.createdAt = kvargs["createdAt"]
        if "error" in kvargs and kvargs["error"] is not None:
            self_.error = kvargs["error"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "isArchived" in kvargs and kvargs["isArchived"] is not None:
            self_.isArchived = kvargs["isArchived"]
        if "isTestRun" in kvargs and kvargs["isTestRun"] is not None:
            self_.isTestRun = kvargs["isTestRun"]
        if "scheduledStartTime" in kvargs and kvargs["scheduledStartTime"] is not None:
            self_.scheduledStartTime = kvargs["scheduledStartTime"]
        if "startTime" in kvargs and kvargs["startTime"] is not None:
            self_.startTime = kvargs["startTime"]
        if "status" in kvargs and kvargs["status"] is not None:
            self_.status = kvargs["status"]
        if "stopTime" in kvargs and kvargs["stopTime"] is not None:
            self_.stopTime = kvargs["stopTime"]
        if "title" in kvargs and kvargs["title"] is not None:
            self_.title = kvargs["title"]
        if "updatedAt" in kvargs and kvargs["updatedAt"] is not None:
            self_.updatedAt = kvargs["updatedAt"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class ScheduleRequestObject:
    """

    Attributes
    ----------
    interval: int
      time in seconds.
    startAt: str
    stopAt: str
    timezone: str
    """

    interval: int = None
    startAt: str = None
    stopAt: str = None
    timezone: str = None

    def __init__(self_, **kvargs):
        if "interval" in kvargs and kvargs["interval"] is not None:
            self_.interval = kvargs["interval"]
        if "startAt" in kvargs and kvargs["startAt"] is not None:
            self_.startAt = kvargs["startAt"]
        if "stopAt" in kvargs and kvargs["stopAt"] is not None:
            self_.stopAt = kvargs["stopAt"]
        if "timezone" in kvargs and kvargs["timezone"] is not None:
            self_.timezone = kvargs["timezone"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class ScheduleResponseObject:
    """

    Attributes
    ----------
    id: str
    interval: int
      time in seconds.
    lastStartedAt: str
    startAt: str
    stopAt: str
    timezone: str
    """

    id: str = None
    interval: int = None
    lastStartedAt: str = None
    startAt: str = None
    stopAt: str = None
    timezone: str = None

    def __init__(self_, **kvargs):
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "interval" in kvargs and kvargs["interval"] is not None:
            self_.interval = kvargs["interval"]
        if "lastStartedAt" in kvargs and kvargs["lastStartedAt"] is not None:
            self_.lastStartedAt = kvargs["lastStartedAt"]
        if "startAt" in kvargs and kvargs["startAt"] is not None:
            self_.startAt = kvargs["startAt"]
        if "stopAt" in kvargs and kvargs["stopAt"] is not None:
            self_.stopAt = kvargs["stopAt"]
        if "timezone" in kvargs and kvargs["timezone"] is not None:
            self_.timezone = kvargs["timezone"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class SettingsObject:
    """

    Attributes
    ----------
    automationsEnabled: bool
    """

    automationsEnabled: bool = None

    def __init__(self_, **kvargs):
        if "automationsEnabled" in kvargs and kvargs["automationsEnabled"] is not None:
            self_.automationsEnabled = kvargs["automationsEnabled"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UsageList:
    """

    Attributes
    ----------
    data: list[UsageObject]
    """

    data: list[UsageObject] = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]" == UsageList.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [UsageObject(**e) for e in kvargs["data"]]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class UsageObject:
    """

    Attributes
    ----------
    automation: AutomationUsageObject
    date: str
      The date for the metric
    name: Literal["runs", "scheduledRun", "triggeredRun", "webhookRuns", "duration", "bandwidthIn", "bandwidthOut"]
      The name for the metric
    value: int
      The value count of the metric
    """

    automation: AutomationUsageObject = None
    date: str = None
    name: Literal[
        "runs",
        "scheduledRun",
        "triggeredRun",
        "webhookRuns",
        "duration",
        "bandwidthIn",
        "bandwidthOut",
    ] = None
    value: int = None

    def __init__(self_, **kvargs):
        if "automation" in kvargs and kvargs["automation"] is not None:
            if (
                type(kvargs["automation"]).__name__
                == UsageObject.__annotations__["automation"]
            ):
                self_.automation = kvargs["automation"]
            else:
                self_.automation = AutomationUsageObject(**kvargs["automation"])
        if "date" in kvargs and kvargs["date"] is not None:
            self_.date = kvargs["date"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "value" in kvargs and kvargs["value"] is not None:
            self_.value = kvargs["value"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Automations:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def get_settings(self) -> SettingsObject:
        """
        Retrieves the automation settings.

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/automations/settings",
            method="GET",
            params={},
            data=None,
        )
        obj = SettingsObject(**response.json())
        obj.auth = self.auth
        return obj

    def set_settings(self, data: SettingsObject) -> SettingsObject:
        """
        Update the automation settings.

        Parameters
        ----------
        data: SettingsObject
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/automations/settings",
            method="PUT",
            params={},
            data=data,
        )
        obj = SettingsObject(**response.json())
        obj.auth = self.auth
        return obj

    def get_usages(self, filter: str, breakdownBy: str = None) -> UsageList:
        """
        Retrieves all automation usage for a tenant
        Limitations:
         * Maximum 1000 metrics per API call

        Parameters
        ----------
        filter: str
          Indicates how the metrics should be filtered
        breakdownBy: str = None
          If specified, result will be broken apart for each automation
        """
        query_params = {}
        if filter is not None:
            query_params["filter"] = filter
        if breakdownBy is not None:
            query_params["breakdownBy"] = breakdownBy
        response = self.auth.rest(
            path="/automations/usage",
            method="GET",
            params=query_params,
            data=None,
        )
        obj = UsageList(**response.json())
        obj.auth = self.auth
        return obj

    def get(self, id: str) -> AutomationDetailResponseObject:
        """
        Retrieves an automation

        Parameters
        ----------
        id: str
          The unique identifier for the automation.
        """
        response = self.auth.rest(
            path="/automations/{id}".replace("{id}", id),
            method="GET",
            params={},
            data=None,
        )
        obj = AutomationDetailResponseObject(**response.json())
        obj.auth = self.auth
        return obj

    def get_automations(
        self,
        filter: str = None,
        limit: int = 100,
        sort: Literal[
            "id",
            "name",
            "runMode",
            "state",
            "createdAt",
            "updatedAt",
            "lastRunAt",
            "lastRunStatus",
            "+id",
            "+name",
            "+runMode",
            "+state",
            "+createdAt",
            "+updatedAt",
            "+lastRunAt",
            "+lastRunStatus",
            "-id",
            "-name",
            "-runMode",
            "-state",
            "-createdAt",
            "-updatedAt",
            "-lastRunAt",
            "-lastRunStatus",
        ] = "id",
    ) -> ListableResource[AutomationListObject]:
        """
        Retrieves a list of automations

        Parameters
        ----------
        filter: str = None
          Allowed filters: name, runMode, lastRunStatus, ownerId
        limit: int = 100
          The number of automations to retrieve.
        sort: Literal["id", "name", "runMode", "state", "createdAt", "updatedAt", "lastRunAt", "lastRunStatus", "+id", "+name", "+runMode", "+state", "+createdAt", "+updatedAt", "+lastRunAt", "+lastRunStatus", "-id", "-name", "-runMode", "-state", "-createdAt", "-updatedAt", "-lastRunAt", "-lastRunStatus"] = "id"
          The field to sort by, with +- prefix indicating sort order. (?query=-name => sort on the name field using descending order)
        """
        query_params = {}
        if filter is not None:
            query_params["filter"] = filter
        if limit is not None:
            query_params["limit"] = limit
        if sort is not None:
            query_params["sort"] = sort
        response = self.auth.rest(
            path="/automations",
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=AutomationListObject,
            auth=self.auth,
            path="/automations",
            query_params=query_params,
        )

    def create(
        self, data: AutomationDetailRequestObject
    ) -> AutomationDetailResponseObject:
        """
        Create an automation

        Parameters
        ----------
        data: AutomationDetailRequestObject
          Automation object to create
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/automations",
            method="POST",
            params={},
            data=data,
        )
        obj = AutomationDetailResponseObject(**response.json())
        obj.auth = self.auth
        return obj



# This is spectacularly generated code by spectacular based on
# Qlik Cloud Services APIs

from __future__ import annotations

from dataclasses import asdict, dataclass
from enum import Enum

from ..auth import Auth, Config


class ConfigReasonCode(Enum):
    Deployment = "deployment"
    Toggle = "toggle"
    License = "license"


@dataclass
class NoteSettingsPutPayload:
    """

    Attributes
    ----------
    snapshotRelations: bool
      pass 'true' to enable the relations api to search notes for the tenant.
    toggledOn: bool
      pass 'true' to enable the note toggle for the tenant, 'false' to disable the toggle (other values are ignore).
    """

    snapshotRelations: bool = None
    toggledOn: bool = None

    def __init__(self_, **kvargs):
        if "snapshotRelations" in kvargs and kvargs["snapshotRelations"] is not None:
            self_.snapshotRelations = kvargs["snapshotRelations"]
        if "toggledOn" in kvargs and kvargs["toggledOn"] is not None:
            self_.toggledOn = kvargs["toggledOn"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class NoteSettingsPutResponse:
    """

    Attributes
    ----------
    snapshotRelations: bool
      'true' if relations api to search notes for the tenant are enabled else false.
    toggleOn: bool
      'true' if the note feature is enabled for this tenant and user otherwise 'false'.
    """

    snapshotRelations: bool = None
    toggleOn: bool = None

    def __init__(self_, **kvargs):
        if "snapshotRelations" in kvargs and kvargs["snapshotRelations"] is not None:
            self_.snapshotRelations = kvargs["snapshotRelations"]
        if "toggleOn" in kvargs and kvargs["toggleOn"] is not None:
            self_.toggleOn = kvargs["toggleOn"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class NotesUserSettings:
    """

    Attributes
    ----------
    available: bool
      'true' if the note feature is enabled for this tenant and user otherwise 'false'.
    lastFetch: str
      The timestamp for the last time this users notes settings were fetched from downstream services.
    reason: Literal["deployment", "toggle", "license"]
      The possible states for the status of notes configuration GET or POST operation
    """

    available: bool = None
    lastFetch: str = None
    reason: ConfigReasonCode = None

    def __init__(self_, **kvargs):
        if "available" in kvargs and kvargs["available"] is not None:
            self_.available = kvargs["available"]
        if "lastFetch" in kvargs and kvargs["lastFetch"] is not None:
            self_.lastFetch = kvargs["lastFetch"]
        if "reason" in kvargs and kvargs["reason"] is not None:
            if (
                type(kvargs["reason"]).__name__
                == NotesUserSettings.__annotations__["reason"]
            ):
                self_.reason = kvargs["reason"]
            else:
                self_.reason = ConfigReasonCode(kvargs["reason"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Notes:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def get_settings(self) -> NotesUserSettings:
        """
        Get the enablement status of the notes feature set for this tenant and user.

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/notes/settings",
            method="GET",
            params={},
            data=None,
        )
        obj = NotesUserSettings(**response.json())
        obj.auth = self.auth
        return obj

    def set_settings(self, data: NoteSettingsPutPayload) -> NoteSettingsPutResponse:
        """
        update the settings

        Parameters
        ----------
        data: NoteSettingsPutPayload
          A JSON payload containing note settings to put.
        """
        if data is not None:
            try:
                data = asdict(data)
            except:
                data = data
        response = self.auth.rest(
            path="/notes/settings",
            method="PUT",
            params={},
            data=data,
        )
        obj = NoteSettingsPutResponse(**response.json())
        obj.auth = self.auth
        return obj



from ._version import __version__  # noqa

# expose from apis
from .apis.Api_Keys import *  # noqa
from .apis.Apps import *  # noqa
from .apis.Audits import *  # noqa
from .apis.Automations import *  # noqa
from .apis.Brands import *  # noqa
from .apis.Collections import *  # noqa
from .apis.Csp_Origins import *  # noqa
from .apis.Data_Files import *  # noqa
from .apis.Extensions import *  # noqa
from .apis.Groups import *  # noqa
from .apis.Identity_Providers import *  # noqa
from .apis.Items import *  # noqa
from .apis.Licenses import *  # noqa
from .apis.Notes import *  # noqa
from .apis.Oauth_Tokens import *  # noqa
from .apis.Qix import *  # noqa
from .apis.Questions import *  # noqa
from .apis.Quotas import *  # noqa
from .apis.Reload_Tasks import *  # noqa
from .apis.Reloads import *  # noqa
from .apis.Roles import *  # noqa
from .apis.Spaces import *  # noqa
from .apis.Tenants import *  # noqa
from .apis.Themes import *  # noqa
from .apis.Transports import *  # noqa
from .apis.Users import *  # noqa
from .apis.Web_Integrations import *  # noqa
from .apis.Webhooks import *  # noqa

# SDK modules
from .auth import Auth  # noqa
from .auth_type import AuthType  # noqa
from .config import Config  # noqa
from .generate_signed_token import generate_signed_token  # noqa
from .qlik import Qlik  # noqa
from .rpc import RequestInterceptor, RequestObject, ResponseInterceptor  # noqa



# This is spectacularly generated code by spectacular based on
# Qlik Cloud Services APIs

from __future__ import annotations

import io
from dataclasses import asdict, dataclass
from typing import Literal

from ..auth import Auth, Config
from ..listable import ListableResource
from ..utils import get_mime_type


@dataclass
class Brand:
    """
    A brand is a collection of assets for applying custom branding. Only a single brand can be active in a tenant.

    Attributes
    ----------
    active: bool
    createdAt: str
      The UTC timestamp when the brand was created.
    createdBy: str
      ID of a user that created the brand.
    description: str
    files: list[BrandFile]
      Collection of resources that make up the brand.
    id: str
    name: str
    updatedAt: str
      The UTC timestamp when the brand was last updated.
    updatedBy: str
      ID of a user that last updated the brand.
    """

    active: bool = None
    createdAt: str = None
    createdBy: str = None
    description: str = None
    files: list[BrandFile] = None
    id: str = None
    name: str = None
    updatedAt: str = None
    updatedBy: str = None

    def __init__(self_, **kvargs):
        if "active" in kvargs and kvargs["active"] is not None:
            self_.active = kvargs["active"]
        if "createdAt" in kvargs and kvargs["createdAt"] is not None:
            self_.createdAt = kvargs["createdAt"]
        if "createdBy" in kvargs and kvargs["createdBy"] is not None:
            self_.createdBy = kvargs["createdBy"]
        if "description" in kvargs and kvargs["description"] is not None:
            self_.description = kvargs["description"]
        if "files" in kvargs and kvargs["files"] is not None:
            if all(
                f"list[{type(e).__name__}]" == Brand.__annotations__["files"]
                for e in kvargs["files"]
            ):
                self_.files = kvargs["files"]
            else:
                self_.files = [BrandFile(**e) for e in kvargs["files"]]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "name" in kvargs and kvargs["name"] is not None:
            self_.name = kvargs["name"]
        if "updatedAt" in kvargs and kvargs["updatedAt"] is not None:
            self_.updatedAt = kvargs["updatedAt"]
        if "updatedBy" in kvargs and kvargs["updatedBy"] is not None:
            self_.updatedBy = kvargs["updatedBy"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class BrandFile:
    """
    Represents one of the assets used as part of the brand. These include logos, favicons, and some styles.

    Attributes
    ----------
    contentType: str
    eTag: str
    id: Literal["logo", "favIcon", "styles"]
    path: str
    """

    contentType: str = None
    eTag: str = None
    id: Literal["logo", "favIcon", "styles"] = None
    path: str = None

    def __init__(self_, **kvargs):
        if "contentType" in kvargs and kvargs["contentType"] is not None:
            self_.contentType = kvargs["contentType"]
        if "eTag" in kvargs and kvargs["eTag"] is not None:
            self_.eTag = kvargs["eTag"]
        if "id" in kvargs and kvargs["id"] is not None:
            self_.id = kvargs["id"]
        if "path" in kvargs and kvargs["path"] is not None:
            self_.path = kvargs["path"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class BrandPatch:
    """
    A JSON Patch document as defined in https://datatracker.ietf.org/doc/html/rfc6902.

    Attributes
    ----------
    op: Literal["add", "remove", "replace"]
      The operation to be performed.
    path: Literal["/name", "/description"]
      The path for the given resource field to patch.
    value: str
      The value to be used for this operation.
    """

    op: Literal["add", "remove", "replace"] = None
    path: Literal["/name", "/description"] = None
    value: str = None

    def __init__(self_, **kvargs):
        if "op" in kvargs and kvargs["op"] is not None:
            self_.op = kvargs["op"]
        if "path" in kvargs and kvargs["path"] is not None:
            self_.path = kvargs["path"]
        if "value" in kvargs and kvargs["value"] is not None:
            self_.value = kvargs["value"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class BrandsList:
    """
    A collection of brands.

    Attributes
    ----------
    data: list[Brand]
    links: BrandsListLinks
    """

    data: list[Brand] = None
    links: BrandsListLinks = None

    def __init__(self_, **kvargs):
        if "data" in kvargs and kvargs["data"] is not None:
            if all(
                f"list[{type(e).__name__}]" == BrandsList.__annotations__["data"]
                for e in kvargs["data"]
            ):
                self_.data = kvargs["data"]
            else:
                self_.data = [Brand(**e) for e in kvargs["data"]]
        if "links" in kvargs and kvargs["links"] is not None:
            if type(kvargs["links"]).__name__ == BrandsList.__annotations__["links"]:
                self_.links = kvargs["links"]
            else:
                self_.links = BrandsListLinks(**kvargs["links"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class BrandsListLinks:
    """

    Attributes
    ----------
    next: Link
    prev: Link
    self: Link
    """

    next: Link = None
    prev: Link = None
    self: Link = None

    def __init__(self_, **kvargs):
        if "next" in kvargs and kvargs["next"] is not None:
            if type(kvargs["next"]).__name__ == BrandsListLinks.__annotations__["next"]:
                self_.next = kvargs["next"]
            else:
                self_.next = Link(**kvargs["next"])
        if "prev" in kvargs and kvargs["prev"] is not None:
            if type(kvargs["prev"]).__name__ == BrandsListLinks.__annotations__["prev"]:
                self_.prev = kvargs["prev"]
            else:
                self_.prev = Link(**kvargs["prev"])
        if "self" in kvargs and kvargs["self"] is not None:
            if type(kvargs["self"]).__name__ == BrandsListLinks.__annotations__["self"]:
                self_.self = kvargs["self"]
            else:
                self_.self = Link(**kvargs["self"])
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class Link:
    """

    Attributes
    ----------
    href: str
      URL of a resource request.
    """

    href: str = None

    def __init__(self_, **kvargs):
        if "href" in kvargs and kvargs["href"] is not None:
            self_.href = kvargs["href"]
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


@dataclass
class NoActiveBrand:
    """
    Empty object inferring lack of active branding.

    Attributes
    ----------
    """

    def __init__(self_, **kvargs):
        for k0, v in kvargs.items():
            k = k0.replace("-", "_")
            if k not in getattr(self_, "__annotations__", {}):
                self_.__setattr__(k, v)


class Brands:
    def __init__(self, config: Config) -> None:
        self.config = config
        self.auth = Auth(config)

    def get_active(self) -> any:
        """
        Retrieves the current active brand

        Parameters
        ----------
        """
        response = self.auth.rest(
            path="/brands/active",
            method="GET",
            params={},
            data=None,
        )
        obj = NoActiveBrand(**response.json())
        obj.auth = self.auth
        return obj

    def activate(self, brand_id: str) -> Brand:
        """
        Activates a brand
        Sets the brand active and de-activates any other active brand. If the brand is already active, no action is taken.

        Parameters
        ----------
        brand_id: str
          The brand's unique identifier.
        """
        response = self.auth.rest(
            path="/brands/{brand-id}/actions/activate".replace("{brand-id}", brand_id),
            method="POST",
            params={},
            data=None,
        )
        obj = Brand(**response.json())
        obj.auth = self.auth
        return obj

    def deactivate(self, brand_id: str) -> Brand:
        """
        Deactivates a brand
        Sets the brand so it is no longer active. If the brand is already inactive, no action is taken.

        Parameters
        ----------
        brand_id: str
          The brand's unique identifier.
        """
        response = self.auth.rest(
            path="/brands/{brand-id}/actions/deactivate".replace(
                "{brand-id}", brand_id
            ),
            method="POST",
            params={},
            data=None,
        )
        obj = Brand(**response.json())
        obj.auth = self.auth
        return obj

    def delete_file(
        self, brand_id: str, brand_file_id: Literal["logo", "favIcon", "styles"]
    ) -> None:
        """
        Deletes a specific brand file

        Parameters
        ----------
        brand_id: str
          The brand's unique identifier.
        brand_file_id: Literal["logo", "favIcon", "styles"]
          The unique identifier of a file within a brand.
        """
        self.auth.rest(
            path="/brands/{brand-id}/files/{brand-file-id}".replace(
                "{brand-id}", brand_id
            ).replace("{brand-file-id}", brand_file_id),
            method="DELETE",
            params={},
            data=None,
        )

    def get_file(
        self, brand_id: str, brand_file_id: Literal["logo", "favIcon", "styles"]
    ) -> str:
        """
        Downloads the brand file

        Parameters
        ----------
        brand_id: str
          The brand's unique identifier.
        brand_file_id: Literal["logo", "favIcon", "styles"]
          The unique identifier of a file within a brand.
        """
        response = self.auth.rest(
            path="/brands/{brand-id}/files/{brand-file-id}".replace(
                "{brand-id}", brand_id
            ).replace("{brand-file-id}", brand_file_id),
            method="GET",
            params={},
            data=None,
            stream=True,
        )
        return response

    def create_file(
        self,
        brand_id: str,
        brand_file_id: Literal["logo", "favIcon", "styles"],
        file: io.BufferedReader = None,
    ) -> BrandFile:
        """
        Creates a brand file

        Parameters
        ----------
        brand_id: str
          The brand's unique identifier.
        brand_file_id: Literal["logo", "favIcon", "styles"]
          The unique identifier of a file within a brand.
        file: str = None
          The path and name of a file to upload.
        """
        files_dict = {}
        files_dict["file"] = ("file", file, get_mime_type(file))
        response = self.auth.rest(
            path="/brands/{brand-id}/files/{brand-file-id}".replace(
                "{brand-id}", brand_id
            ).replace("{brand-file-id}", brand_file_id),
            method="POST",
            params={},
            data=None,
            files=files_dict,
        )
        obj = BrandFile(**response.json())
        obj.auth = self.auth
        return obj

    def set_file(
        self,
        brand_id: str,
        brand_file_id: Literal["logo", "favIcon", "styles"],
        file: io.BufferedReader = None,
    ) -> BrandFile:
        """
        Updates existing file

        Parameters
        ----------
        brand_id: str
          The brand's unique identifier.
        brand_file_id: Literal["logo", "favIcon", "styles"]
          The unique identifier of a file within a brand.
        file: str = None
          A file to upload.
        """
        files_dict = {}
        files_dict["file"] = ("file", file, get_mime_type(file))
        response = self.auth.rest(
            path="/brands/{brand-id}/files/{brand-file-id}".replace(
                "{brand-id}", brand_id
            ).replace("{brand-file-id}", brand_file_id),
            method="PUT",
            params={},
            data=None,
            files=files_dict,
        )
        obj = BrandFile(**response.json())
        obj.auth = self.auth
        return obj

    def delete(self, brand_id: str) -> None:
        """
        Deletes a specific brand

        Parameters
        ----------
        brand_id: str
          The brand's unique identifier.
        """
        self.auth.rest(
            path="/brands/{brand-id}".replace("{brand-id}", brand_id),
            method="DELETE",
            params={},
            data=None,
        )

    def get(self, brand_id: str) -> Brand:
        """
        Retrieves a specific brand

        Parameters
        ----------
        brand_id: str
          The brand's unique identifier.
        """
        response = self.auth.rest(
            path="/brands/{brand-id}".replace("{brand-id}", brand_id),
            method="GET",
            params={},
            data=None,
        )
        obj = Brand(**response.json())
        obj.auth = self.auth
        return obj

    def patch(self, brand_id: str, BrandPatchData: list[BrandPatch]) -> None:
        """
        Patches a brand

        Parameters
        ----------
        brand_id: str
          The brand's unique identifier.
        BrandPatchData: list[BrandPatch]
        """
        data = [asdict(elem) for elem in BrandPatchData]
        self.auth.rest(
            path="/brands/{brand-id}".replace("{brand-id}", brand_id),
            method="PATCH",
            params={},
            data=data,
        )

    def get_brands(
        self,
        endingBefore: str = None,
        limit: int = 5,
        sort: Literal[
            "id",
            "+id",
            "-id",
            "createdAt",
            "+createdAt",
            "-createdAt",
            "updatedAt",
            "+updatedAt",
            "-updatedAt",
        ] = "-id",
        startingAfter: str = None,
    ) -> ListableResource[Brand]:
        """
        Lists all brand entries for a tenant

        Parameters
        ----------
        endingBefore: str = None
          Cursor to previous.
        limit: int = 5
          Maximum number of brands to retrieve.
        sort: Literal["id", "+id", "-id", "createdAt", "+createdAt", "-createdAt", "updatedAt", "+updatedAt", "-updatedAt"] = "-id"
          Field to sort by, prefixed with -/+ to indicate the order.
        startingAfter: str = None
          Cursor to the next page.
        """
        query_params = {}
        if endingBefore is not None:
            query_params["endingBefore"] = endingBefore
        if limit is not None:
            query_params["limit"] = limit
        if sort is not None:
            query_params["sort"] = sort
        if startingAfter is not None:
            query_params["startingAfter"] = startingAfter
        response = self.auth.rest(
            path="/brands",
            method="GET",
            params=query_params,
            data=None,
        )
        return ListableResource(
            response=response.json(),
            cls=Brand,
            auth=self.auth,
            path="/brands",
            query_params=query_params,
        )

    def create(
        self,
        logo: io.BufferedReader = None,
        name: str = None,
        styles: io.BufferedReader = None,
        favIcon: io.BufferedReader = None,
        description: str = None,
    ) -> Brand:
        """
        Creates a new brand

        Parameters
        ----------
        logo: str = None
          The path and name of a JPG or PNG file that will be adjusted to fit in a 'box' measuring 109px in width and 62 px in height while maintaining aspect ratio. Maximum size of 300 KB, but smaller is recommended.
        name: str = None
          Name of the brand.
        styles: str = None
          The path and name of a JSON file to define brand style settings. Maximum size is 100 KB. This property is not currently operational.
        favIcon: str = None
          The path and name of a properly formatted ICO file. Maximum size is 100 KB.
        description: str = None
          Description of the brand.
        """
        files_dict = {}
        files_dict["logo"] = ("logo", logo, get_mime_type(logo))
        files_dict["styles"] = ("styles", styles, get_mime_type(styles))
        files_dict["favIcon"] = ("favIcon", favIcon, get_mime_type(favIcon))
        if name is not None:
            files_dict["name"] = name
        if description is not None:
            files_dict["description"] = description
        response = self.auth.rest(
            path="/brands", method="POST", params={}, data=None, files=files_dict
        )
        obj = Brand(**response.json())
        obj.auth = self.auth
        return obj
